<?php
/**
 * Clase con funciones estáticas genéricas del sistema SWPHP
 *
 * Jose Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2012-01-13
 *
 * SIMPLE WEB PHP EDICION ADMINISTRADA
 *
 * inc/classes/SWPHP.class.php
 *
 */
abstract class SWPHP
{
	/**
	 * @var type SWPHP_Breadcrumb
	 */
	public static $breadcrumb;
	/**
	 * Muestra o devuelve el contenido de una variable. Útil sobretodo para depurar código.
	 *
	 * @param mixed $var
	 */
	static function _VarExport($var, $return=false)
	{
		$aux = '<pre>'.var_export($var, true).'</pre>';
		if($return)
		{
			return $aux;
		}
		echo $aux;
	}
	/**
	 * Finaliza la ejecución del script mostrando el contenido de una variable. Útil para depurar código.
	 *
	 * @param mixed $var
	 */
	static function _Die($var)
	{
		die(SWPHP::_VarExport($var, true));
	}
	/**
	 * Añade un mensaje a la pila de mensajes a mostrar
	 *
	 * @param string $texto
	 * @param string $clase
	 * @param mixed $concepto
	 * @param string $isocode
	 */
	static function _Msg($texto, $clase='info', $concepto=false, $isocode=null)
	{
		global $swphp_mensajes;

		if($concepto !== false)
		{
			$texto = l($texto, $concepto === true ? 'theme' : $concepto, $isocode);
		}

		$swphp_mensajes[] = array('clase'=>$clase, 'texto'=>$texto);
	}
	/**
	 * Obtiene la pila actual de mensajes en HTML
	 *
	 * @param bool $limpiar
	 * @return string
	 */
	static function _GetMsgs($limpiar=false)
	{
		global $swphp_mensajes;

		$html = '';
		foreach($swphp_mensajes as $msg)
		{
			$html .= sprintf('<div class="%s">%s</div>', $msg['clase'], $msg['texto']);
		}

		if($limpiar)
		{
			$swphp_mensajes = array();
		}

		return $html;
	}
	/**
	 * Si el modo debug está activado añade un mensaje a la pila de mensajes a mostrar
	 *
	 * @param string $texto
	 * @param string $clase
	 * @param mixed $concepto
	 * @param string $isocode
	 */
	static function _MsgDbg($texto, $clase='info', $concepto=false, $isocode=null)
	{
		if(defined('SWPHP_DEBUG') && SWPHP_DEBUG)
		{
			SWPHP::_Msg($texto, $clase, $concepto, $isocode);
			//return SWPHP::_Msg($texto, $clase, $concepto, $isocode);
		}

		//return false;
	}
	/**
	 * Obtiene un array con los idiomas definidos (todos o sólo los operativos)
	 *
	 * @param bool $todos
	 * @return array
	 */
	static function _GetIdiomas($todos=false)
	{
		global $db;

		$filtro = $todos ? '' : "WHERE activo = '1' OR id = '".SWPHP_IDIOMA_POR_DEFECTO."'";

		$arr = array();
		$rs = DB_ResultSet::Factory($db->GetExtension());
		$rs->Set = $db->Execute("SELECT id, nombre, codigoiso, codigolang FROM ".SWPHP_DBPREFIX."idiomas {$filtro}");
		while($reg = $rs->FetchAssoc())
		{
			$arr[$reg['id']] = $reg;
		}

		return $arr;
	}
	/**
	 * Obtiene el valor de un campo de una tabla relacionada con idiomas
	 *
	 * @param string $campo
	 * @param int $que_id
	 * @param string $campoid
	 * @param string $que
	 * @param int $idioma_id
	 * @return mixed
	 */
	static function _GetIdiomaValue($campo, $que_id, $campoid='pagina_id', $que='paginas', $idioma_id=null)
	{
		global $db;

		if(is_null($idioma_id))
		{
			$idioma_id = $_SESSION['SWPHP_IDIOMA_ACTUAL'];
		}

		$val = $db->GetValueQuery("SELECT `{$campo}` FROM `".SWPHP_DBPREFIX."{$que}_idiomas` WHERE `{$campoid}` = '{$que_id}' AND idioma_id = '{$idioma_id}'", false);
		if($idioma_id != SWPHP_IDIOMA_POR_DEFECTO && (is_null($val) || $val == ''))
		{
			$val = SWPHP::_GetIdiomaValue($campo, $que_id, $campoid, $que, SWPHP_IDIOMA_POR_DEFECTO);
		}

		return $val;
	}
	/**
	 * Obtiene el enlace amigable a una página
	 *
	 * @param int $pagina_id
	 * @return string
	 */
	static function _GetHrefPagina($pagina_id, $idioma_id=null)
	{
		global $db;

		if(is_null($idioma_id))
		{
			$idioma_id = $_SESSION['SWPHP_IDIOMA_ACTUAL'];
		}

		$pagina = $db->GetRow("SELECT p.id, p.padre_id FROM ".SWPHP_DBPREFIX."paginas AS p WHERE p.id = '{$pagina_id}'", true, false);
		$href[] = SWPHP::_GetIdiomaValue('friendly', $pagina_id, 'pagina_id', 'paginas', $idioma_id);
		while($pagina['padre_id'] > 0)
		{
			$pagina = $db->GetRow("SELECT p.id, p.padre_id FROM ".SWPHP_DBPREFIX."paginas AS p WHERE p.id = '{$pagina['padre_id']}'", true, false);
			$href[] = SWPHP::_GetIdiomaValue('friendly', $pagina['id'], 'pagina_id', 'paginas', $idioma_id);
		}

		$href = implode('/', array_reverse($href)).'/';

		if($href && substr($href, 0, 1) == '/')
		{
			$href = substr($href, 1);
		}

		return $href;
	}
	/**
	 * Obtiene el conjunto de opciones para el menú pricipal
	 *
	 * @return array
	 */
	static function _GetMenuPrincipal($id=0)
	{
		global $db;

		$menu = array();
		$rows = $db->GetRows("SELECT m.id, m.pagina_id, m.abriraparte, m.clase FROM ".SWPHP_DBPREFIX."menuprincipal AS m WHERE m.padre_id = '{$id}' AND m.activo = 1 ORDER BY m.peso ASC, m.id ASC", true, false);
		foreach($rows as $row)
		{
			if($row['pagina_id'] > 0)
			{
				$aux = $db->GetRow("SELECT pi.textoenlace, pi.title, p.icono, p.estilo_id, e.clave AS estilo FROM ".SWPHP_DBPREFIX."paginas AS p JOIN ".SWPHP_DBPREFIX."paginas_idiomas AS pi ON p.id = pi.pagina_id AND pi.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}' LEFT JOIN ".SWPHP_DBPREFIX."estilos AS e ON e.id = p.estilo_id WHERE p.id = '{$row['pagina_id']}'");
				$aux['href'] = SWPHP::_GetHrefPagina($row['pagina_id']);
			}
			else
			{
				$aux = $db->GetRow("SELECT mi.url AS href, mi.textoenlace, mi.title, m.icono, m.estilo_id, e.clave AS estilo FROM ".SWPHP_DBPREFIX."menuprincipal AS m JOIN ".SWPHP_DBPREFIX."menuprincipal_idiomas AS mi ON m.id = mi.menuprincipal_id AND mi.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}' LEFT JOIN ".SWPHP_DBPREFIX."estilos AS e ON e.id = m.estilo_id WHERE m.id = '{$row['id']}'");
			}

			$aux['abriraparte'] = $row['abriraparte'];
            $aux['clase'] = $row['clase'];
			$aux['on'] = (!empty($aux['href']) && defined('SWPHP_PAGINA_ACTUAL') && substr(SWPHP_PAGINA_ACTUAL, 0, strlen($aux['href'])) == $aux['href']) || (empty($aux['href']) && defined('SWPHP_PAGINA_ACTUAL') && SWPHP_PAGINA_ACTUAL == '');
			$aux['hijos'] = SWPHP::_GetMenuPrincipal($row['id']);

			$menu[] = $aux;
		}

		return $menu;
	}
	/**
	 * Obtiene el título y el conjunto de opciones para el menú contextual correspondiente a una página
	 *
	 * @param int $pagina_id
	 * @return array
	 */
	static function _GetMenuContextual($pagina_id=0)
	{
		global $db;

		while($pagina_id > 0 && $db->GetValueQuery("SELECT conmenucontextual FROM ".SWPHP_DBPREFIX."paginas WHERE id = '{$pagina_id}'", false) == 0)
		{
			$pagina_id = $db->GetValueQuery("SELECT padre_id FROM ".SWPHP_DBPREFIX."paginas WHERE id = '{$pagina_id}'", false);
		}

		return SWPHP::_GetMenu($pagina_id, "enmenucontextual = '1' AND activo = '1'");
	}
	/**
	 * Obtiene datos y el conjunto de opciones (páginas hijas) correspondientes a una página
	 *
	 * @param int $pagina_id
	 * @param string $filtroextra
	 * @param string $orderby
	 * @return array
	 */
	static function _GetMenu($pagina_id=0, $filtroextra="activo = 1", $orderby="peso, id")
	{
		global $db;

		$row = SWPHP::_GetOpcion($pagina_id);

		$row['opciones'] = SWPHP::_GetHijas($pagina_id, $filtroextra, $orderby);

		return $row;
	}
	/**
	 * Obtiene datos correspondientes a una página, útiles para construir una opción (enlace).
	 *
	 * @param int $pagina_id
	 * @return array
	 */
	static function _GetOpcion($pagina_id=0)
	{
		global $db;

		$row = $db->GetRow("SELECT p.id, pi.title, pi.textoenlace, pi.titulo FROM ".SWPHP_DBPREFIX."paginas AS p JOIN ".SWPHP_DBPREFIX."paginas_idiomas AS pi ON p.id = pi.pagina_id AND pi.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}' WHERE p.id = '{$pagina_id}'");

		$row['href'] = SWPHP::_GetHrefPagina($pagina_id);
		$row['on'] = SWPHP_PAGINA_ACTUAL == $row['href'];
		$row['enruta'] = SWPHP_PAGINA_ACTUAL ? ($row['href'] && strpos(SWPHP_PAGINA_ACTUAL, $row['href']) === 0) : ($row['href'] ? true : false);

		return $row;
	}
	/**
	 * Obtiene datos correspondientes a una página, útiles para construir una opción (enlace).
	 *
	 * @param int $pagina_id
	 * @return array
	 */
	static function _GetOpcionNext($pagina_id=0)
	{
		global $db;

		$pagina = $db->GetRow("SELECT id, padre_id, peso FROM ".SWPHP_DBPREFIX."paginas WHERE id = '{$pagina_id}'");
		if(!empty($pagina))
		{
			$next_id = $db->GetValueQuery("SELECT id FROM ".SWPHP_DBPREFIX."paginas WHERE padre_id = '{$pagina['padre_id']}' AND peso >= '{$pagina['peso']}' AND id > '{$pagina['id']}' ORDER BY peso, id");

			return SWPHP::_GetOpcion($next_id);
		}

		return null;
	}
	/**
	 * Obtiene las opciones para un menú de navegación a partir de las hijas de una página.
	 * Si $full entonces devuelve el árbol completo, si no $full entonces sólo despliega la opción seleccionada.
	 *
	 * @param int $padre_id
	 * @param bool $full
	 * @return array
	 */
	static function _GetNavegacion($padre_id=0, $full=true)
	{
		$navegacion = array();
		$hijas = SWPHP::_GetHijas($padre_id);
		foreach($hijas as $hija)
		{
			$navegacion[] = $full || strpos(SWPHP_PAGINA_ACTUAL, $hija["href"]) === 0 ? SWPHP::_GetMenu($hija["id"]) : SWPHP::_GetOpcion($hija["id"]);
		}

		return $navegacion;
	}
	/**
	 * Obtiene el conjunto de páginas hijas de una página
	 *
	 * @param int $pagina_id
	 * @param string $filtroextra
	 * @param string $orderby
	 * @param bool $recursivo
	 * @return array
	 */
	static function _GetHijas($pagina_id=0, $filtroextra="activo = '1'", $orderby="peso, id", $recursivo=false)
	{
		global $db;

		$filtro = "WHERE padre_id = '{$pagina_id}'";
		if($filtroextra)
		{
			$filtro .= " AND {$filtroextra}";
		}

		if($orderby)
		{
			$filtro .= " ORDER BY {$orderby}";
		}

		$paginas = $db->GetRows("SELECT p.id, pi.title, pi.textoenlace, pi.descripcion, p.icono, p.fechayhoraultima FROM ".SWPHP_DBPREFIX."paginas AS p JOIN ".SWPHP_DBPREFIX."paginas_idiomas AS pi ON p.id = pi.pagina_id AND pi.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}' {$filtro}");
		for($c=0,$size=sizeof($paginas); $c<$size; $c++)
		{
			$paginas[$c]['href'] = SWPHP::_GetHrefPagina($paginas[$c]['id']);
			$paginas[$c]['on'] = (SWPHP_PAGINA_ACTUAL == $paginas[$c]['href']);

			if($recursivo)
			{
				$paginas[$c]['opciones'] = SWPHP::_GetHijas($paginas[$c]['id'], $filtroextra, $orderby, true);
			}
		}

		return $paginas;
	}
	/**
	 * Obtiene el html correspondiente a un icono, o sólo su ruta si $params se establece como false
	 *
	 * @param string $icon
	 * @param int $width
	 * @param mixed $params
	 * @param string $alt
	 * @return string
	 */
	static function _Icon($icon, $width='', $params=array(), $alt=null)
	{
		$img = SWPHP::_GetIconoPath($icon, $width);

		if($params === false)
		{
			return $img;
		}

		$img = "<img src=\"{$img}\"";
		$valids = array('border', 'style', 'class', 'alt', 'id', 'longdesc', 'onclick', 'ondblclick', 'onmouseover', 'onmouseout', 'title', 'usemap');
		foreach($params as $key=>$value)
		{
			if(in_array($key, $valids))
			{
				$img .= " {$key}=\"{$value}\"";
			}
		}
		$pos = strpos($icon, '.');
		$obligados = array('alt'=>($pos ? substr($icon, 0, $pos) : $icon));
		foreach($obligados as $key=>$value)
		{
			if(!array_key_exists($key, $params))
			{
				$img .= " {$key}=\"{$value}\"";
			}
		}
		$img .= " />";

		return $img;
	}
	/**
	 * Obtiene la ruta del icono indicado para el tamaño indicado, o en su lugar el icono por defecto
	 *
	 * @param string $icono
	 * @param int $width
	 * @return string
	 */
	static function _GetIconoPath($icono, $width, $alt=null)
	{
		if(!in_array($width, array(16,24,32,48,64,128)))
		{
			$width = SWPHP_DEFAULT_ICON_WIDTH;
		}

		$path = SWPHP_THEME_DIR."img/icons/{$width}x{$width}/{$icono}";

		if(is_file($path))
		{
			return $path;
		}

		if(is_null($alt))
		{
			switch($icono)
			{
				//TODO: más equivalencias
				case 'view.png':
					$alt = 'search.png';
					break;
				case 'check.png':
					$alt = 'accept.png';
					break;
				case 'delete.png':
					$alt = 'remove.png';
					break;
				case 'remove.png':
					$alt = 'delete.png';
					break;
				default:
					$alt = SWPHP_DEFAULT_ICON;
					break;
			}
		}

		return SWPHP::_GetIconoPath($alt, $width);
	}
	/**
	 * Obtiene el conjunto de iconos disponibles
	 *
	 * @param int $width
	 * @param bool $fullpath
	 * @return array
	 */
	static function _GetIconos($width=16, $fullpath=false)
	{
		if(!in_array($width, array(16,24,32,48,64,128)))
		{
			$width = 16;
		}
		$dir = SWPHP_THEME_DIR."img/icons/{$width}x{$width}/";
		$currentdir = getcwd();
		chdir($dir);
		$handle = opendir(".");
		$files = array();
		while($file = readdir($handle))
		{
			if(is_file($file) && getExtension($file) == 'png')
			{
				$files[] = $fullpath ? $dir.$file : $file;
			}
		}
		chdir($currentdir);
		sort($files);
		return $files;
	}
	/**
	 * Obtiene el html correspondiente a una banderita de idioma, o sólo su ruta si $params se establece como false
	 * $lang es el código iso del idioma
	 *
	 * @param string $lang
	 * @param mixed $params
	 * @return string
	 */
	static function _LangFlag($lang, $params=array())
	{
		$img = "img/lang/{$lang}.jpg";
		if(!is_file($img))
		{
			$img = "img/lang/default.jpg";
		}

		if($params === false)
		{
			return $img;
		}

		$img = "<img src=\"{$img}\"";
		$valids = array('border', 'style', 'class', 'alt', 'id', 'longdesc', 'onclick', 'ondblclick', 'onmouseover', 'onmouseout', 'title', 'usemap');
		foreach($params as $key=>$value)
		{
			if(in_array($key, $valids))
			{
				$img .= " {$key}=\"{$value}\"";
			}
		}
		$obligados = array('alt');
		foreach($obligados as $key)
		{
			if(!array_key_exists($key, $params))
			{
				$img .= " {$key}=\"\"";
			}
		}
		$img .= " />";

		return $img;
	}
	/**
	 * Obtiene el conjunto de elementos del localizador
	 *
	 * @param int $pagina_id
	 * @return array
	 *
	 * @deprecated since v1.0.7
	 */
	static function _GetLocalizador($pagina_id)
	{
		global $db;

		$primero = true;
		$localizador = array();
		while($pagina_id > 0)
		{
			$reg = $db->GetRow("SELECT p.id, pi.titulo, p.padre_id FROM ".SWPHP_DBPREFIX."paginas AS p JOIN ".SWPHP_DBPREFIX."paginas_idiomas AS pi ON p.id = pi.pagina_id AND pi.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}' WHERE p.id = '{$pagina_id}'");
			$pagina_id = $reg['padre_id'];
			if(!$primero)
			{
				//$reg['href'] = SWPHP::_GetHrefPagina($reg['id']);
				$localizador[] = $reg;
			}
			$primero = false;
		}

		return array_reverse($localizador);
	}
	/**
	 * Elimina uno o varios registros de una tabla
	 *
	 * @param string $table
	 * @param mixed $ids
	 * @param string $que
	 * @param bool $redirigir
	 * @param string $sqlOnSuccess
	 * @param string $phpOnSuccess
	 * @return bool
	 */
	static function _OpDel($table, $ids, $que="registro", $redirigir=true, $sqlOnSuccess="", $phpOnSuccess="")
	{
		global $db;

		$ok = false;

		if($db->Execute(buildQuery_Delete($table, $ids)))
		{
			if($db->AffectedRows() > 0)
			{
				$ok = true;

				SWPHP::_Msg(sprintf("%s eliminad%s correctamente", ucfirst($que), substr($que, strlen($que)-1) == "a" ? "a" : "o"), "check");

				if($sqlOnSuccess)
				{
					$db->Execute($sqlOnSuccess);
				}

				if($phpOnSuccess)
				{
					eval($phpOnSuccess);
				}
			}
		}
		else
		{
			SWPHP::_Msg(sprintf("No se ha podido eliminar el %s", $que), "error");
		}

		if($redirigir)
		{
			SWPHP::_Redirigir();
		}

		return $ok;
	}
	/**
	 * Activa/Desactiva un campo bandera (valor cero o uno) de uno o varios registros de una tabla
	 *
	 * @param string $table
	 * @param mixed $ids
	 * @param bool $activo
	 * @param bool $redirigir
	 * @param string $campo
	 */
	static function _OpOnOff($table, $ids, $activo, $redirigir=true, $campo="activo")
	{
		SWPHP::_OpValue($table, $ids, $activo?1:0, $redirigir, $campo);
	}
	/**
	 * Asigna un valor a un campo de uno o varios registros de una tabla
	 *
	 * @param string $table
	 * @param mixed $ids
	 * @param mixed $value
	 * @param bool $redirigir
	 * @param string $campo
	 */
	static function _OpValue($table, $ids, $value, $redirigir, $campo)
	{
		global $db;

		$db->Execute(buildQuery_Update(array($campo=>$value), $table, $ids));
		if($redirigir)
		{
			SWPHP::_Redirigir();
		}
	}
	/**
	 * Cambia el valor de un campo de tipo 0/1 para el registro indicado
	 *
	 * @param string $table
	 * @param mixed $ids
	 * @param string $campo
	 * @param array $otrosCampos
	 * @return bool
	 */
	static function _Switch($table, $ids, $campo='activo', $otrosCampos=array())
	{
		global $db;

		if(is_array($otrosCampos) && sizeof($otrosCampos) > 0)
		{
			$query = "UPDATE `".$table."` SET ";
			$keys = array_keys($otrosCampos);
			for($c=0,$size=sizeof($otrosCampos); $c<$size; $c++)
			{
				if($c > 0)
					$query .= ", ";
				$query .= "`".$keys[$c]."`='".addslashes($otrosCampos[$keys[$c]])."'";
			}
			$query .= ", `{$campo}` = IF(`{$campo}`=1,0,1)";
			$query .= DB_Connection::ids2where($ids);
		}
		else
		{
			$query = "UPDATE {$table} SET `{$campo}` = IF(`{$campo}`=1,0,1) ".DB_Connection::ids2where($ids);
		}

		return ($ids && $db->Execute($query) && $db->AffectedRows() == 1);
	}
	/**
	 * Redirige a una determinada url
	 *
	 * @param string $url
	 * @param bool $limpiarOp
	 */
	static function _Redirigir($url=null, $limpiarOp=true)
	{
		global $swphp_mensajes, $swphp_config;

        //Guardamos la pila actual de mensajes antes de redirigir
        $_SESSION["swphp"]["mensajes"] = $swphp_mensajes;

        ///Forzamos guardar la sesión
        session_commit();

        header("Location: ".SWPHP::_LimpiarOp(is_null($url) ? (isset($_SESSION["REQUEST_URI"]) ? $_SESSION["REQUEST_URI"] : $swphp_config['url']) : $url));
        exit;
	}
	/**
	 * Quita determinadas variables op de la query de una url
	 *
	 * @param string $url
	 * @param array $ops
	 * @return string
	 */
	static function _LimpiarOp($url, $ops=array("reg","upd","del","on","off"))
	{
		$arr1 = array();
		$arr2 = array();
		foreach($ops as $op)
		{
			$arr1[] = "op={$op}";
			$arr1[] = "&op={$op}";
		}
		return str_replace($arr1, $arr2, $url);
	}
	/**
	 * Ejecuta una sentencia SQL
	 *
	 * @param string $query
	 * @param string $mensajeOk
	 * @param string $mensajeErr
	 * @param bool $mostrarErrorSQL
	 * @param string $logSqlErrTo
	 * @return bool
	 */
	static function _ExecuteQuery($query, $mensajeOk, $mensajeErr, $mostrarErrorSQL=true, $logSqlErrTo="")
	{
		global $db;
		if($return = $db->Execute($query))
		{
			if(strlen($mensajeOk) > 0)
			{
				SWPHP::_Msg($mensajeOk, "check");
			}
		}
		else
		{
			$sqlErr = $db->Error();
			if($mostrarErrorSQL)
			{
				$mensajeErr .= "<br />$sqlErr";
			}
			if($logSqlErrTo != "")
			{
				SWPHP::_Log("ERROR $query => $sqlErr", $logSqlErrTo);
			}
			if(strlen($mensajeErr) > 0)
			{
				SWPHP::_Msg($mensajeErr, "error");
			}
		}
		return $return;
	}
	/**
	 * Añade una línea con el mensaje pasado al archivo .log de nombre $tipo (si no existe lo crea). Estos archivos se almacenan en la carpeta "log"
	 *
	 * @param string $mensaje
	 * @param string $tipo
	 */
	static function _Log($mensaje, $tipo="info")
	{
		$fp = fopen(sprintf(SWPHP_LOG_DIR.'%s.%s.log', $tipo, date("Y-m")), 'a');
		$script = basename($_SERVER['PHP_SELF']);
		fwrite($fp, date("Y-m-d H:i:s")."\t[{$tipo}]\t{$script}\t-\t{$mensaje}\n");
		fclose($fp);
	}
	/**
	 * Guarda un archivo subido en $dir con el nombre original o con $newname si se proporciona,
	 * en cualquier caso siempre como nombre seguro de archivo.
	 * Sólo admite las extensiones especificadas en $exts que será una cadena de extensiones separadas por coma.
	 * Devuelve el nombre del archivo guardado en caso de éxito y false en cualquier otro caso.
	 * $mostrarmensaje:
	 * -1 = Excepto si es no se ha subido el archivo
	 *  0 = No mostrar mensaje
	 *  1 = Mostrar mensaje siempre
	 *
	 * @param string $formitemname
	 * @param string $dir
	 * @param string $exts
	 * @param bool $renameifexists
	 * @param string $newname
	 * @param bool $mostrarenlace
	 * @param int $mostrarmensaje
	 * @return mixed
	 */
	static function _SubirArchivo($formitemname='archivo', $dir=null, $exts=SWPHP_ALLOWED_UPLOAD_FILE_EXTENSIONS, $renameifexists=true, $newname="", $mostrarenlace=true, $mostrarmensaje=1)
	{
		if(isset($_FILES[$formitemname]))
		{
			global $swphp_config;

			$file = $_FILES[$formitemname];
			$exts = explode(',', $exts);

			switch($file["error"])
			{
				case UPLOAD_ERR_OK:
					if(is_uploaded_file($file["tmp_name"]))
					{
						$ext = getExtension($file["name"]);
						if(in_array($ext, $exts))
						{
							include_once(SWPHP_COMMON_DIR.'classes/MyCoder.class.php');

							if(is_null($dir))
							{
								$dir = SWPHP_UPLOADS_DIR;

								if(isset($_POST['ubicacion']) && is_dir(SWPHP_UPLOADS_DIR.$_POST['ubicacion']))
								{
									$dir .= $_POST['ubicacion'];
								}
							}

							$dir = ponerBarra($dir);

							$nombre = $newname ? $newname : $file["name"];
							$ext = getExtension($nombre);
							$nombre = getName($nombre);
							$nombreguardado = MyCoder::SecureFileName("{$nombre}.{$ext}");
							$destino = $dir.$nombreguardado;

							if((filesize($file["tmp_name"])/1024) <= SWPHP_MAX_UPLOAD_FILE_SIZE)
							{
								if($renameifexists)
								{
									$c = 1;
									while(file_exists($destino))
									{
										$destino = $dir.MyCoder::SecureFileName("{$nombre}-{$c}.{$ext}");
										$c++;
									}
								}

								if(move_uploaded_file($file["tmp_name"], $destino))
								{
									@chmod($destino, 0775);

									if($mostrarmensaje)
									{
										$msg = l("Se ha recibido el archivo correctamente y se ha guardado como").sprintf(" <b>%s</b>",	$nombreguardado);
										if($mostrarenlace)
										{
											$base_url = $swphp_config['url'];
											$enlace = ($base_url[strlen($base_url)-1] == "/") ? $base_url.$destino : $base_url."/".$destino;
											$msg .= sprintf("<p>Enlace al archivo: <a href=\"%s\" class=\"external\">%s</a></p><p class=\"mini\">Ruta completa: %s</p>", $enlace, wordwrap($destino,80,"<br />",1),	$enlace);
										}
										SWPHP::_Msg($msg, "check");
									}

									return basename($destino);
								}
								else
								{
									SWPHP::_Msg(l('No se ha podido grabar el archivo').sprintf(" (%s)", $formitemname), "error");
								}
							}
							else
							{
								SWPHP::_Msg(l('El archivo no debe ocupar más de').sprintf(" %d Kb (%s)", SWPHP_MAX_UPLOAD_FILE_SIZE, $formitemname), "error");
							}
						}
						else
						{
							SWPHP::_Msg(l('El archivo tiene una extensión no admitida').sprintf(" %s (%s)<br />".l('Sólo se admiten:')." %s", $ext, $formitemname, implode(", ",$exts)), "error");
						}
					}
					else
					{
						SWPHP::_Msg(l('No se ha recibido el archivo').sprintf(" (%s)", $formitemname), "aviso");
					}
					break;
				case UPLOAD_ERR_INI_SIZE:
			    case UPLOAD_ERR_FORM_SIZE:
			        SWPHP::_Msg(l('El archivo es demasiado grande')." ({$formitemname})", "error");
			        break;
			    case UPLOAD_ERR_PARTIAL:
			        SWPHP::_Msg(l('El archivo sólo se subió parcialmente')." ({$formitemname})", "error");
			        break;
			    case UPLOAD_ERR_NO_FILE:
			        if($mostrarmensaje > 0)
			        {
						SWPHP::_Msg(l('No se ha subido el archivo')." ({$formitemname})", "aviso");
			        }
			        break;
			    case UPLOAD_ERR_NO_TMP_DIR:
			        SWPHP::_Msg(l('No existe el directorio temporal de subida')." ({$formitemname})", "error");
			        break;
			    case UPLOAD_ERR_CANT_WRITE:
			        SWPHP::_Msg(l('No se pudo grabar el archivo')." ({$formitemname})", "error");
			        break;
			    case UPLOAD_ERR_EXTENSION:
			        SWPHP::_Msg(l('Extensión no admitida para subir el archivo')." ({$formitemname})", "error");
			        break;
			}
		}

		return false;
	}
	/**
	 * Muestra el formulario de autenticación o información sobre el usuario si ya está autenticado.
	 * $subtemplate es la variable que indica la plantilla a incluir como swphp_templatecontenido
	 *
	 * @param string $subtemplate
	 * @param bool $assign
	 */
	static function _PedirLogin(&$subtemplate, $assign=false)
	{
		global $swphp_pagina, $smarty;

		$swphp_pagina->SetValues(
			'',
			'',
			'Autenticación de usuarios',
			'login, autenticacion, panel, administracion'
			);

		$subtemplate = 'paraincluir/login.tpl';

		SWPHP::_HabilitarLogin();

		if($assign)
		{
			$smarty->assign('swphp_templatecontenido', $subtemplate);
		}
	}
	/**
	 * Asigna a smarty las variables necesarias para que funcione el formulario de login
	 *
	 */
	static function _HabilitarLogin($url=false)
	{
		global $smarty;

		$_SESSION['swphp']['loginkey'] = GenerateRandomKey(9);

		$smarty->assign('loginkey', $_SESSION['swphp']['loginkey']);
		$smarty->assign('logincontrol', SWPHP::_Encrypt($_SESSION['swphp']['loginkey']));
		if(!empty($url))
		{
			$smarty->assign('loginurl', $url);
		}
	}
	/**
	 * Cierra la sessión de usuario
	 *
	 * @param string $redirectTo
	 * @param bool $destroySession
	 */
	static function _Logout($redirectTo=null, $destroySession=false)
	{
		$_SESSION['user'] = array('acutenticado'=>false);

		if($destroySession)
		{
			session_destroy();
		}

		if(!is_null($redirectTo))
		{
			SWPHP::_Redirigir($redirectTo);
		}
	}
	/**
	 * Encripta una cadena
	 *
	 * @param string $str
	 * @return string
	 */
	static function _Encrypt($str)
	{
		return md5($str.SWPHP_FRASE_SECRETA);
	}
	/**
	 * Comprueba si existe y se puede escribir sobre un directorio
	 *
	 * @param string $dir
	 * @param bool $createIfNotExists
	 * @param bool $dieOnFail
	 * @param bool $level
	 * @return bool
	 */
	static function _CheckDirWritable($dir, $createIfNotExists=true, $dieOnFail=false, $level=0)
	{
		$dirs = array_filter(explode('/', $dir));
		$size = sizeof($dirs);
		if($level == 0 && $size > 1)
		{
			$return = true;
			$path = '';
			for($c=0; $c<$size; $c++)
			{
				$path .= "{$dirs[$c]}/";
				if($c >= $level)
				{
					$return = SWPHP::_CheckDirWritable($path, $createIfNotExists, $dieOnFail, $level+1) && $return;
				}
			}

			return $return;
		}
		else
		{
			$path = $dir;

			if(file_exists($path))
			{
				if(is_writable($path))
				{
					return true;
				}
				elseif($level == 0)
				{
					$msg = "Se necesitan permisos de escritura sobre el directorio <b>{$path}</b>";
					if($dieOnFail)
					{
						die($msg);
					}
					else
					{
						SWPHP::_Msg($msg, 'aviso');
					}
				}
			}
			elseif($createIfNotExists)
			{
				SWPHP::_MkDir($path, true, 0775);

				return SWPHP::_CheckDirWritable($path, false, $dieOnFail);
			}
			else
			{
				$msg = "No se encuentra el directorio <b>{$path}</b>";
				if($dieOnFail)
				{
					die($msg);
				}
				else
				{
					SWPHP::_Msg($msg, 'aviso');
				}
			}
		}

		return false;
	}
	/**
	 * Obtiene un objeto SWPHP_Paginador
	 *
	 * @param string $key
	 * @param int $rpp
	 * @return SWPHP_Paginador
	 */
	static function _CrearPaginador($key=null, $rpp=50)
	{
		include_once(SWPHP_COMMON_DIR.'classes/SWPHP_Paginador.class.php');
		include_once(SWPHP_COMMON_DIR.'classes/SqlSelectQuery.class.php');

		global $db, $swphp_administracion;

		$paginador = new SWPHP_Paginador($key, $rpp);
		$paginador->img_dir = SWPHP_THEME_DIR.'img/';
		$paginador->img_buscar = SWPHP::_Icon('search.png', 16, false);

		if($swphp_administracion)
		{
			$paginador->SetParam('action', SWPHP_ADMIN_DIR.'index.php');
		}
		else
		{
			$paginador->SetParam('action', 'index.php');
		}

		return $paginador;
	}
	/**
	 * Obtiene un objeto DB_Paginador para ser usado desde el frontend
	 *
	 * @param int $max_regs
	 * @param bool $conDB
	 * @return DB_Paginador
	 * @deprecated Utilizar _CrearPaginador
	 */
	static function _CrearPaginadorFrontend($max_regs=25, $conDB=true)
	{
		include_once(SWPHP_COMMON_DIR.'classes/DB_Paginador.class.php');

		global $db;

		$paginador = new DB_Paginador($max_regs, $conDB ? $db : null);
		$paginador->action = 'index.php';
		$paginador->img_dir = SWPHP_THEME_DIR.'img/';
		$paginador->img_buscar = 'icons/16x16/search.png';

		return $paginador;
	}
	/**
	 * Obtiene un objeto DB_Paginador para ser usado desde el panel de administración
	 *
	 * @param int $max_regs
	 * @return DB_Paginador
	 * @deprecated Utilizar _CrearPaginador
	 */
	static function _CrearPaginadorAdmin($max_regs=25)
	{
		include_once(SWPHP_COMMON_DIR.'classes/DB_Paginador.class.php');

		global $db;

		$paginador = new DB_Paginador($max_regs, $db);
		$paginador->action = SWPHP_ADMIN_DIR.'index.php';
		$paginador->img_dir = SWPHP_THEME_DIR.'img/';
		$paginador->img_buscar = 'icons/16x16/search.png';

		return $paginador;
	}
	/**
	 * Envía un email y graba el resultado en un archivo de log
	 *
	 * @param string $dest
	 * @param string $asunto
	 * @param string $mensaje
	 * @param array $archivos
	 * @param string $recip
	 * @param string $recip_nombre
	 * @param string &$error
	 * @param string $replyTo
	 * @param string $replyName
	 * @return bool
	 */
	static function _EnviarEmail($dest, $asunto, $mensaje, $archivos=null, $recip='', $recip_nombre='', &$error, $replyTo='', $replyName='')
	{
		include_once(SWPHP_COMMON_DIR."phpmailer/class.phpmailer.php");
		include_once(SWPHP_COMMON_DIR."phpmailer/language/phpmailer.lang-es.php");
		include_once(SWPHP_COMMON_DIR."classes/Html2Text.class.php");

		$config = SWPHP::_GetConfigValues('configuracion_email', false);

		if(!$recip)
		{
			$recip = $config['emailenvios'];
		}

		if(!$recip_nombre)
		{
			$recip_nombre = $config['nombreenvios'];
		}

		$em = new PHPMailer();
		$em->CharSet = SWPHP_CHARSET;
		$em->From = $recip;
		$em->FromName = $recip_nombre;
		if($replyTo)
		{
			$em->AddReplyTo($replyTo, $replyName);
			$mensaje .= '<br /><br />'.l('Email de respuesta:').' '.$replyTo;
		}
		if(is_array($dest))
		{
			foreach($dest as $emailto)
			{
				$em->AddBCC($emailto);
			}
		}
		else
		{
			$em->AddAddress($dest);
		}
		$em->Subject = $asunto;

		if(is_array($archivos) && ($size=sizeof($archivos)) > 0)
		{
			for($c=0; $c<$size; $c++)
			{
				$file = $archivos[$c];
				if(file_exists($file))
				{
					$em->AddAttachment($file, basename($file));
				}
			}
			$em->AttachAll();
		}

		if(defined('SWPHP_TEMPLATES_DIR') && is_file(SWPHP_TEMPLATES_DIR.'email.tpl'))
		{
			global $smarty, $swphp_config;

			$smarty->assign('email_contenido', $mensaje);
			$smarty->assign('swphp_config', $swphp_config);

			$mensaje = $smarty->fetch('email.tpl');
		}

		$html2text = new Html2Text($mensaje);
		$em->Body = $mensaje;
		$em->AltBody = $html2text->get_text();

		if($config['usar_smtp'])
		{
			$em->IsSMTP();
			$em->SMTPAuth = true;
			$em->Host = $config['smtp_host'];
			$port = $config['smtp_port'];
			if($port)
			{
				$em->Port = $port;
			}
			$em->Username = $config['smtp_user'];
			$em->Password = $config['smtp_pass'];
		}
		else
		{
			$em->IsMail();
		}

		if($em->Send())
		{
			SWPHP::_Log("OK\t$dest\t$asunto", "sendmail");
			return true;
		}
		else
		{
			$error = str_replace('Language string failed to load: ', '', $em->ErrorInfo);
			SWPHP::_Log("ERROR\t$dest\t$asunto ***ERROR*** {$error}", "sendmail");
			return false;
		}
	}
	/**
	 * Obtiene un registro de configuración
	 *
	 * @param string $variable
	 * @param string $concepto
	 * @param string $campos
	 * @param bool $htmlentities
	 * @param string $table
	 * @return mixed
	 */
	static function _GetConfigRow($variable, $concepto, $campos='*', $htmlentities=true, $table='config')
	{
		global $db;

		return $db->GetRow("SELECT {$campos} FROM ".SWPHP_DBPREFIX."{$table} WHERE variable = '{$variable}' AND concepto = '{$concepto}'", true, $htmlentities);
	}
	/**
	 * Obtiene un conjunto de registros de configuración
	 *
	 * @param string $concepto
	 * @param string $campos
	 * @param bool $htmlentities
	 * @param string $table
	 * @return mixed
	 */
	static function _GetConfigRows($concepto, $campos='*', $htmlentities=true, $table='config')
	{
		global $db;

		return $db->GetRows("SELECT {$campos} FROM ".SWPHP_DBPREFIX."{$table} WHERE variable = '{$variable}' AND concepto = '{$concepto}'", true, $htmlentities);
	}
	/**
	 * Obtiene un valor de configuración
	 *
	 * @param string $variable
	 * @param string $concepto
	 * @param bool $htmlentities
	 * @param string $table
	 * @return mixed
	 */
	static function _GetConfigValue($variable, $concepto, $htmlentities=true, $table='config', $default=null)
	{
		global $db;
		$rs = DB_ResultSet::Factory($db->GetExtension());
		if($rs->Set = $db->Execute("SELECT valor FROM ".SWPHP_DBPREFIX."{$table} WHERE variable = '{$variable}' AND concepto = '{$concepto}'"))
			if($reg = $rs->FetchRow())
				return $htmlentities ? SWPHP::_HtmlEntities($reg[0]) : $reg[0];
		return $default;
	}
	/**
	 * Obtiene un conjunto de valores de configuración
	 *
	 * @param string $concepto
	 * @param bool $htmlentities
	 * @param string $table
	 * @return array
	 */
	static function _GetConfigValues($concepto, $htmlentities=true, $table='config', $defaults=array())
	{
		global $db;

		$data = array();
		$rs = DB_ResultSet::Factory($db->GetExtension());
		if($rs->Set = $db->Execute("SELECT variable, valor FROM ".SWPHP_DBPREFIX."{$table} WHERE concepto = '{$concepto}'"))
			while($reg = $rs->FetchRow())
				$data[$reg[0]] = $htmlentities ? SWPHP::_HtmlEntities($reg[1]) : $reg[1];
		foreach($defaults as $key=>$val)
		{
			if(!isset($data[$key]))
			{
				$data[$key] = $val;
			}
		}

		//Desde la versión 1.1.3 del módulo contacto,
		//hay una tabla de localizaciones,
		//donde se crea un registro con los datos de la única localización hasta ahora,
		//aquí, si es el concepto contacto,
		//completamos con los datos de ese registro para tener compatibilidad hacia atrás.
		if($concepto == 'contacto')
		{
			$keys = array('direccion1', 'direccion2', 'localidad', 'provincia', 'codigopostal', 'telefono', 'fax', 'movil', 'movil2', 'googlemaps', 'horario1', 'horario2');
			$arr = $db->GetValuesQuery("SELECT * FROM `".SWPHP_DBPREFIX."localizaciones` ORDER BY `peso` ASC, `id` ASC LIMIT 1", $htmlentities, true);
			foreach($arr as $key=>$val)
			{
				if(is_array($val))
				{
					foreach($val as $key2=>$val2)
					{
						if(in_array($key2, $keys))
						{
							$data[$key2] = $val2;
						}
					}

					break;
				}
				elseif(in_array($key, $keys))
				{
					$data[$key] = $val;
				}
			}
		}

		return $data;
	}
	/**
	 * Obtiene varios valores de configuración agrupados por concepto
	 *
	 * @param string $concepto
	 * @param bool $htmlentities
	 * @param string $table
	 * @return array
	 */
	static function _GetConfigValuesMulti($concepto, $htmlentities=true, $table='config')
	{
		global $db;
		$conceptos = $db->GetValuesQuery("SELECT DISTINCT concepto FROM ".SWPHP_DBPREFIX."{$table} WHERE concepto LIKE '{$concepto}%'");
		for($c=0,$size=sizeof($conceptos); $c<$size; $c++)
		{
			$conceptos[$c] = array(
				"concepto"=>$conceptos[$c],
				"data"=>$db->GetRows("SELECT variable, valor FROM ".SWPHP_DBPREFIX."{$table} WHERE concepto = '{$conceptos[$c]}'", true, $htmlentities)
				);
		}
		return $conceptos;
	}
	/**
	 * Obtiene un conjunto de valores de configuración preparado para la clase FormBuilder
	 *
	 * @param string $concepto
	 * @param bool $htmlentities
	 * @param string $table
	 * @return array
	 */
	static function _GetConfigValuesToFormBuilder($concepto, $permiso_id=SWPHP_PERMISO_ACCESOTOTAL, $htmlentities=true, $table='config')
	{
		global $db;
		$conceptos = $db->GetValuesQuery("SELECT DISTINCT concepto FROM ".SWPHP_DBPREFIX."{$table} WHERE concepto LIKE '{$concepto}%' ORDER BY concepto", false);
		for($c=0,$size=sizeof($conceptos); $c<$size; $c++)
		{
			$conceptos[$c] = array(
				"concepto"=>$conceptos[$c],
				"data"=>$db->GetRows(sprintf("SELECT id, variable, valor, concepto, info, texto, IF('%u' = %u, %u, IF('%u' < permiso_id, '%u', permiso_id)) AS permiso_id FROM ".SWPHP_DBPREFIX."%s WHERE concepto = '%s' ORDER BY id", $permiso_id, SWPHP_PERMISO_ACCESOTOTAL, SWPHP_PERMISO_ACCESOTOTAL, $permiso_id, $permiso_id, $table, $conceptos[$c]), true, $htmlentities),
				);
		}
		return $conceptos;
	}
	/**
	 * Obtiene valores de configuración
	 *
	 * @param string $concepto
	 * @param bool $htmlentities
	 * @param string $table
	 * @return array
	 */
	static function _GetConfigAssoc($concepto, $htmlentities=true, $table='config')
	{
		global $db;

		$config = array();
		$rows = $db->GetRows("SELECT * FROM ".SWPHP_DBPREFIX."{$table} WHERE concepto LIKE '{$concepto}%'", true, $htmlentities);
		foreach($rows as $row)
		{
			if($row['concepto'] == $concepto)
			{
				$config[$row['variable']] = $row['valor'];
			}
			else
			{
				$config[str_replace($concepto.'_', '', $row['concepto'])][$row['variable']] = $row['valor'];
			}
		}
		return $config;
	}
	/**
	 * Indica si existe un valor de configuración
	 *
	 * @param string $variable
	 * @param string $concepto
	 * @param string $table
	 * @return bool
	 */
	static function _ExistsConfigValue($variable, $concepto='configuracion', $table='config')
	{
		global $db;

		return $db->Exists($variable, "variable", SWPHP_DBPREFIX.$table, "WHERE concepto = '{$concepto}'");
	}
	/**
	 * Establece un valor a una variable en la tabla de configuraciones $table
	 *
	 * @param string $variable
	 * @param mixed $valor
	 * @param string $concepto
	 * @param bool $addslashes
	 * @param string $table
	 * @param bool $conOjo
	 * @return bool
	 */
	static function _SetConfigValue($variable, $valor, $concepto, $addslashes=true, $table='config', $conOjo=false)
	{
		global $db;

		if($addslashes)
		{
			$variable = addslashes($variable);
		}

		if(SWPHP::_ExistsConfigValue($variable, $concepto, $table))
		{
			if($conOjo && $db->GetValueQuery("SELECT permiso_id FROM ".SWPHP_DBPREFIX.$table." WHERE variable = '{$variable}' AND concepto = '{$concepto}'") < SWPHP_PERMISO_LECTURAYESCRITURA)
			{
				$query = false;
			}
			else
			{
				$query = "UPDATE ".SWPHP_DBPREFIX."{$table} SET valor = '{$valor}' WHERE variable = '{$variable}' AND concepto = '{$concepto}'";
			}
		}
		else
		{
			$query = "INSERT INTO ".SWPHP_DBPREFIX."{$table}(variable,valor,concepto) VALUES('{$variable}','{$valor}','{$concepto}')";
		}

		return $query ? ($db->Execute($query) && $db->AffectedRows() != -1) : 0;
	}
	/**
	 * Establece varios valores de configuración
	 *
	 * @param array $data
	 * @param string $concepto
	 * @param bool $addslashes
	 * @param string $table
	 * @param bool $conOjo
	 * @return bool
	 */
	static function _SetConfigValues($data, $concepto, $addslashes=true, $table='config', $conOjo=false)
	{
		$return = true;

		foreach($data as $key=>$value)
		{
			$return = SWPHP::_SetConfigValue($key, $value, $concepto, $addslashes, $table, $conOjo) && $return;
		}

		return $return;
	}
	/**
	 * Si aún no existe una variable en la tabla de configuraciones $table, la crea y le establece un valor
	 *
	 * @param string $variable
	 * @param mixed $valor
	 * @param string $concepto
	 * @param string $info
	 * @param string $texto
	 * @param int $permiso_id
	 * @param bool $sobreEscribirValorSiExiste
	 * @param string $table
	 * @return bool
	 */
	static function _CreateConfigValue($variable, $valor, $concepto, $info=null, $texto=null, $permiso_id=SWPHP_PERMISO_LECTURAYESCRITURA, $sobreEscribirValorSiExiste=false, $table='config')
	{
		global $db;

		$data = array(
			'variable'=>$variable,
			'valor'=>$valor,
			'concepto'=>$concepto,
			'info'=>$info ? $info : '',
			'texto'=>$texto ? $texto : '',
			'permiso_id'=>intval($permiso_id),
			);

		if($db->Exists($variable, "variable", SWPHP_DBPREFIX.$table, sprintf("WHERE concepto = '%s'", addcslashes($concepto,"'"))))
		{
			return $sobreEscribirValorSiExiste ? ($db->Execute(buildQuery_Update($data, SWPHP_DBPREFIX.$table, array('variable'=>$variable, 'concepto'=>$concepto))) && $db->AffectedRows() != -1) : true;
		}

		$db->Execute(buildQuery_Insert($data, SWPHP_DBPREFIX.$table)) && $db->AffectedRows() != -1;

        return $db->AffectedRows() > 0;
	}
	/**
	 * Si no existen, crea y establece varios valores de configuración
	 *
	 * @param array $data
	 * @param string $concepto
	 * @param string $table
	 * @return bool
	 */
	static function _CreateConfigValues($data, $concepto, $table='config')
	{
		$return = true;

		foreach($data as $key=>$value)
		{
			$return = SWPHP::_CreateConfigValue($key, $value, $concepto, null, null, 2, $table) && $return;
		}

		return $return;
	}
	/**
	 * Elimina una variable de configuración
	 *
	 * @param string $variable
	 * @param string $concepto
	 * @param string $table
	 * @return bool
	 */
	static function _DeleteConfigValue($variable, $concepto, $table='config')
	{
		global $db;

		return $db->Execute(sprintf("DELETE FROM %s WHERE concepto = '%s' AND variable = '%s'", SWPHP_DBPREFIX.$table, $concepto, $variable)) !== false;
	}
	/**
	 * Elimina varios valores de configuración
	 *
	 * @param array $data
	 * @param string $concepto
	 * @param string $table
	 * @return bool
	 */
	static function _DeleteConfigValues($data, $concepto, $table='config')
	{
		$return = true;

		foreach($data as $key=>$value)
		{
			$return = SWPHP::_DeleteConfigValue($key, $concepto, $table) && $return;
		}

		return $return;
	}
	/**
	 * Elimina todos los valores de configuración de un concepto
	 *
	 * @param string $concepto
	 * @param string $table
	 * @return bool
	 */
	static function _DeleteAllConfigValues($concepto, $table='config')
	{
		global $db;

		return $db->Execute(sprintf("DELETE FROM %s WHERE concepto = '%s'", SWPHP_DBPREFIX.$table, $concepto)) !== false;
	}
	/**
     * Hace un include del archivo o archivos especificados en $incluir si se encuentran en la ruta $dir
     *
     * @param mixed $incluir
     * @param string $dir
     * @return bool
     */
    static function _Include($incluir, $dir="")
    {
        global $db, $swphp_config, $swphp_pagina, $swphp_mod_clave, $swphp_module_dir, $swphp_module_templates_dir, $swphp_permiso_id,
        	$swphp_portada, $swphp_administracion,
        	$swphp_lang, $swphp_idiomas,
        	$smarty, $xajax;

    	if(is_array($incluir))
        {
            $ok = true;
        	foreach($incluir as $inc)
            {
                $ok &= SWPHP::_Include($inc, $dir);
            }
            return $ok;
        }
        else
        {
            if($dir && substr($dir, strlen($dir)-1) != '/')
            {
                $dir .= '/';
            }

            $incluir = $dir.$incluir;
            if(is_file($incluir))
            {
                include($incluir);
                return true;
            }
        }

        return false;
    }
	/**
	 * Genera una nueva clave a utilizar para un registro de una tabla con un campo que hace de clave unívoca
	 * El parámetro $formato es opcional, si se indica se ignoran los demás y se creará una clave siguiendo el formato.
	 * El formato es una cadena formada sólo por caracteres A, a, 1
	 * A = Una letra mayúscula
	 * a = Una letra minúscula
	 * 1 = Un dígito
	 * [otro] = Un carácter aleatorio (letra mayúscula o minúscula, o dígito)
	 * \ = Carácter de escape, hace que el siguiente carácter sea interpretado como constante
	 *
	 * EJEMPLOS DE FORMATO - RESULTADO:
	 * AAA111	-	VUS293
	 * AAA-111	-	VUSc293
	 * AAA\-111	-	VUS-293
	 * AAA\\111	-	VUS\293
	 * Aaa1		-	Vus2
	 *
	 * @param string $tabla
	 * @param string $campo
	 * @param int $longitud
	 * @param bool $conletrasmin
	 * @param bool $conletrasmay
	 * @param bool $connumeros
	 * @param string $formato
	 * @return string
	 */
	static function _GenerarClaveTabla($tabla, $campo="clave", $longitud=9, $conletrasmin=true, $conletrasmay=true, $connumeros=true, $formato='')
	{
		global $db;

		$clave = GenerateRandomKey($longitud, $conletrasmin, $conletrasmay, $connumeros, $formato);
		while($db->Exists($clave, $campo, $tabla))
		{
			$clave = GenerateRandomKey($longitud, $conletrasmin, $conletrasmay, $connumeros, $formato);
		}

		return $clave;
	}
	/**
	 * Procesa una cadena en busca de plugins, etc. y los aplica devolviendo el código html resultante
	 *
	 * @param string $contenido
	 * @return string
	 */
	static function _ProcesarContenido($contenido)
	{
		$contenido = SWPHP::_ProcesarContenidoSegunPHP($contenido);
		$contenido = SWPHP::_ProcesarContenidoSegunPlugins($contenido);

		return $contenido;
	}
	static function _ProcesarContenidoSegunPHP($contenido)
	{
		//Procesamos el contenido en busca de php
		$aux = "";
		$pos = strpos($contenido, "[swphp:php]");
		while($pos !== false)
		{
			$aux .= substr($contenido, 0, $pos);

			$pos += 11;
			$code = substr($contenido, $pos, strpos($contenido, "[/swphp]", $pos)-$pos);
			$fuente = "[swphp:php]{$code}[/swphp]";
			ob_start();
			eval("{$code};");
			$aux .= ob_get_clean();

			$contenido = substr($contenido, strpos($contenido, $fuente) + strlen($fuente));
			$pos = strpos($contenido, "[swphp:php]");
		}
		$aux .= $contenido;

		return $aux;
	}
	static function _ProcesarContenidoSegunPlugins($contenido)
	{
		//Procesamos el contenido en busca de plugins (han sido cargados desde comunini.inc.php)
		$aux = "";
		$pos = strpos($contenido, "[swphp:plugin]");
		while($pos !== false)
		{
			$aux .= substr($contenido, 0, $pos);

			$pos += 14;
			$code = substr($contenido, $pos, strpos($contenido, "[/swphp]", $pos)-$pos);
			$func = $func = "swphp_plugin_".html_entity_decode($code); //"swphp_plugin_{$code}";
			$fuente = "[swphp:plugin]{$code}[/swphp]";
			$aux .= function_exists(substr($func, 0, strpos($func,"("))) ? eval("return {$func};") : $fuente;

			$contenido = substr($contenido, strpos($contenido, $fuente) + strlen($fuente));
			$pos = strpos($contenido, "[swphp:plugin]");
		}
		$aux .= $contenido;

		/*
		//Procesamos el contenido en busca de plugins (se cargan aquí según se necesitan)
		$aux = "";
		$pos = strpos($contenido, "[swphp:plugin]");
		while($pos !== false)
		{
			$aux .= substr($contenido, 0, $pos);

			$pos += 14;
			$plugin = substr($contenido, $pos, strpos($contenido, "_", $pos)-$pos);

			$swphp_plugin_dir = sprintf(SWPHP_PLUGINS_DIR."%s/", $plugin);
			$file = "{$swphp_plugin_dir}functions.inc.php";
			if(is_dir($swphp_plugin_dir) && is_file($file))
			{
				include_once($file);

				$code = substr($contenido, $pos, strpos($contenido, "[/swphp]", $pos)-$pos);
				$func = "swphp_plugin_{$code}";
				$fuente = "[swphp:plugin]{$code}[/swphp]";
				$aux .= function_exists(substr($func, 0, strpos($func,"("))) ? eval("return {$func};") : $fuente;
			}

			$contenido = substr($contenido, strpos($contenido, $fuente) + strlen($fuente));
			$pos = strpos($contenido, "[swphp:plugin]");
		}
		$aux .= $contenido;
		*/

		return $aux;
	}
	/**
	 * Obtiene el identificador del registro marcado como por defecto de los existentes en una tabla
	 *
	 * @param string $tabla
	 * @param string $campoid
	 * @param string $campopordefecto
	 * @param book $anadirprefijotabla
	 * @return mixed
	 */
	static function _GetPorDefecto($tabla, $campoid="id", $campopordefecto="pordefecto", $anadirprefijotabla=true)
	{
		global $db;

		return $db->GetValueQuery("SELECT `{$campoid}` FROM `".($anadirprefijotabla ? SWPHP_DBPREFIX : "")."{$tabla}` ORDER BY `{$campopordefecto}` DESC, `{$campoid}` ASC LIMIT 1", false);
	}
	/**
	 * Convierte un tamaño de imagen en el formato en como se guarda en configuración, al formato en como se usa en resizeImage
	 *
	 * @param string $size
	 * @return mixed
	 */
	static function _ConfigSize2ArraySize($size)
	{
		$size = strtolower($size);

		$reflen =  strlen($size) - 1;
		$lastchar = substr($size, $reflen);
		switch($lastchar)
		{
			case "h":
				return array(0, substr($size, 0, $reflen));
			case "w":
				return array(substr($size, 0, $reflen), 0);
			default:
				if(strpos($size, "x"))
				{
					return explode("x", $size);
				}

				return $size;
		}

		//Por si acaso
		return $size;
	}
	/**
	 * Si el theme actual cuenta con una versión personalizada de la plantilla que se indica,
	 * devuelve la ruta relativa a la misma,
	 * en caso contrario devuelve la ruta relativa a la plantilla original del módulo.
	 * De esta forma, se puede personalizar por completo el aspecto de un módulo que utilice esta función
	 * para asignar las plantillas, como por ejemplo el módulo Contacto.
	 * Si no se indica $mod_clave, se entiende el módulo actualmente cargado.
	 * Si $relativo_a_templatesdir es true, la ruta relativa es desde SWPHP_TEMPLATES_DIR,
	 * si es false, es desde el directorio raiz (esto segundo es útil para los plugins).
	 *
	 * @param string $template
	 * @param string $mod_clave
	 * @param bool $relativo_a_templatesdir
	 * @return string
	 */
	static function _GetModuleTemplate($template, $mod_clave=null, $relativo_a_templatesdir=true)
	{
		global $swphp_mod_clave, $swphp_module_templates_dir;

		if(is_null($mod_clave))
		{
			$mod_clave = $swphp_mod_clave;
		}

		$aux = "modulos/{$mod_clave}/{$template}";

		if($relativo_a_templatesdir)
		{
			return is_file(SWPHP_TEMPLATES_DIR.$aux) ? $aux : $swphp_module_templates_dir.$template;
		}

		return is_file(SWPHP_TEMPLATES_DIR.$aux) ? SWPHP_TEMPLATES_DIR.$aux : SWPHP_MODULES_DIR."{$mod_clave}/tpl/{$template}";
	}
	/**
	 * Carga los archivos de idioma del módulo indicado
	 *
	 * @param string $clave
	 * @param string $isocode
	 */
	static function _LoadModuleLang($clave, $isocode=null)
	{
		if(is_null($isocode))
		{
			$isocode = $_SESSION['SWPHP_IDIOMA_ACTUAL_CODIGOISO'];
		}

		SWPHP::_Include(SWPHP_MODULES_DIR.$clave.'/lang/'.$isocode.'/index.php');
		SWPHP::_Include(SWPHP_THEME_DIR.'lang/modulos/'.$clave.'/'.$isocode.'/index.php');
	}
	/**
	 * Obtiene un array asociativo con los registros de una tabla,
	 * pudiendo especificar el campo que hace de clave y el que hace de valor.
	 * Se puede filtrar con $filter.
	 * Se puede añadir un elemento con $option.
	 *
	 * @param string $table
	 * @param string $text_field
	 * @param string $index_field
	 * @param string $filter
	 * @param array $option
	 * @param bool $addcomilla
	 * @return array
	 */
	static function _Rows2Assoc($table, $text_field, $index_field="id", $filter="", $option=null)
	{
		global $db;

		return $db->GetArrayForHTMLSelect($index_field, $text_field, $table, $filter, $option);
	}
	/**
	 * Obtiene un array asociativo con los registros de una tabla,
	 * pudiendo especificar el campo que hace de clave y el que hace de valor.
	 * Se puede añadir un elemento con $option.
	 *
	 * @param string $query
	 * @param array $option
	 * @return array
	 */
	static function _Rows2AssocQuery($query, $option=null)
	{
		global $db;

		return $db->GetArrayForHTMLSelectQuery($query, $option);
	}
	/**
	 * Obtiene un array asociativo con los registros de una tabla,
	 * siguiendo una jerarquía de padre-hijo,
	 * pudiendo especificar el campo que hace de clave, el que hace de valor, y el campo que indica el padre.
	 * Se puede filtrar con $filter.
	 * Se puede añadir un elemento con $option.
	 *
	 * @param int $current_id
	 * @param int $nivel
	 * @param int $padre_id
	 * @return array
	 */
	static function _Rows2AssocTree($table, $text_field, $index_field="id", $padre_field="padre_id", $filter="", $option=null, $current_id=0, $nivel=0, $padre_id=0)
	{
		global $db;

		if(empty($filter))
		{
			$filter = "1";
		}

		$rs = DB_ResultSet::Factory($db->GetExtension());
		$rs->Set = $db->Execute("SELECT {$index_field} AS id, {$text_field} AS nombre FROM {$table} WHERE {$padre_field} = '{$padre_id}' AND {$filter}");
		$array = array();

		if(is_array($option) && is_array($option[0]))
		{
			foreach($option as $opt)
			{
				if(is_array($opt) && sizeof($opt) == 2)
				{
					$array[$opt[0]] = $opt[1];
				}
			}
		}
		elseif(is_array($option) && sizeof($option) == 2)
		{
			$array[$option[0]] = $option[1];
		}

		while($reg = $rs->FetchAssoc())
		{
			$pre = "";
			for($c=0; $c<$nivel; $c++)
			{
				$pre .= "- ";
			}
			if($reg["id"] != $current_id)
			{
				$array[$reg["id"]] = $pre.$reg["nombre"];
			}
			if($reg["id"] != $current_id && is_array($subcats = SWPHP::_Rows2AssocTree($table, $text_field, $index_field, $padre_field, $filter, null, $current_id, $nivel+1, $reg["id"])))
			{
				$keys = array_keys($subcats);
				for($c=0,$size=sizeof($keys); $c<$size; $c++)
					if($keys[$c] != $current_id)
						$array[$keys[$c]] = $subcats[$keys[$c]];
			}
		}
		return $array;
	}
	/**
	 * Fuerza la descarga de un archivo si existe, y si no asigna un mensaje de error.
	 *
	 * @param string $filepath
	 * @param string $nombreparaladescarga
	 */
	static function _ForzarDescarga($filepath, $nombreparaladescarga=null)
	{
		if(is_file($filepath))
		{
			$f = fopen($filepath, "rb");
			$content_len = (int)filesize($filepath);
			$content_file = fread($f, $content_len);
			fclose($f);

			$filename = empty($nombreparaladescarga) ? basename($filepath) : $nombreparaladescarga;

			$user_agent = $_SERVER['HTTP_USER_AGENT'];

			@ob_end_clean();
			@ini_set("zlib.output_compression", "Off");
			header("Pragma: public");
			header(sprintf("Last-Modified: %s GMT", gmdate('D, d M Y H:i:s')));
			header("Cache-Control: no-store, no-cache, must-revalidate", false); // HTTP/1.1
			header("Cache-Control: pre-check=0, post-check=0, max-age=0", false); // HTTP/1.1
			header("Cache-Control: private", false);
			header("Content-Transfer-Encoding: none");
			header("Content-Type: application/octet-stream; name=\"{$filename}\"", false); // This should work for IE & Opera
			//header('Content-Type: application/force-download; name="' . $filename . '"', false); //This should work for Non IE/Opera browsers

			if(preg_match("/MSIE 5.5/", $user_agent))
			{
				header("Content-Disposition: filename=\"{$filename}\"");
			}
			else
			{
				header("Content-Disposition: attachment; filename=\"{$filename}\"");
			}

			//header("Content-Disposition: attachment; filename=\"{$filename}\"");
			header("Content-length: {$content_len}");

			die($content_file);
		}
		else
		{
			SWPHP::_Msg("No se encuentra el archivo para su descarga", "error", true);
		}
	}
	/**
	 * Crea una copia de un archivo y redirige a la misma.
	 * Esa copia habrá que borrarla luego, desde un cron o algo,
	 * tiene que tenerlo en cuenta el desarrollador.
	 *
	 * @param string $filepath
	 * @param string $prefijoOnombre
	 * @param string $dirTemp
	 * @param bool $limpiar
	 */
	static function _DescargaTemporal($filepath, $prefijoOnombre="", $dirTemp=SWPHP_TEMP_DIR, $limpiar=true)
	{
		global $swphp_config;

		if($limpiar)
		{
			SWPHP::_LimpiarTemporales($dirTemp);
		}

		if(is_file($filepath))
		{
			$len = strlen($dirTemp);
			if($len > 0 && substr($dirTemp, $len-1) != "/")
			{
				$dirTemp .= "/";
			}

			$file = basename($filepath);

			$len = strlen($prefijoOnombre);
			$prefijo = $len > 3 ? '' : $prefijoOnombre.'_';
			$nombreArchivo = $len > 3 ? '_'.$prefijoOnombre : (($ext = getExtension($file)) ? '.'.$ext : '');

			$random = GenerateRandomKey(6);
			$tempfile = $dirTemp.$prefijo.$random.$nombreArchivo;
			while(is_file($tempfile))
			{
				$random = GenerateRandomKey(6);
				$tempfile = $dirTemp.$prefijo.$random.$nombreArchivo;
			}

			copy($filepath, $tempfile);
			if(is_file($tempfile))
			{
				header("Location: {$swphp_config['url']}{$tempfile}");
				exit;

				//El archivo se queda ahí, luego habrá que borrarlo con un cron o algo
			}
			else
			{
				SWPHP::_Msg("No se ha podido crear el archivo temporal para su descarga", "error", true);
			}
		}
		else
		{
			SWPHP::_Msg("No se encuentra el archivo para su descarga", "error", true);
		}
	}
	/**
	 * Elimina archivos temporales
	 *
	 * @param string $dirTemp
	 * @param int $antiguedad
	 */
	static function _LimpiarTemporales($dirTemp=SWPHP_TEMP_DIR, $antiguedad=3600)
	{
		deleteFiles($dirTemp, null, $antiguedad);
	}
	/**
	 * Sube un registro por peso
	 *
	 * @param int $id
	 * @param string $table
	 * @param string $campoPadre
	 * @param string $campoPeso
	 * @param string $campoId
	 */
	static function _SubirRegistro($id, $table, $campoPadre='padre_id', $campoPeso='peso', $campoId='id')
	{
		global $db;

		if($campoPadre)
		{
			$row = $db->GetRow("SELECT `{$campoPeso}`, `{$campoPadre}` FROM `{$table}` WHERE `{$campoId}` = '{$id}'", true, false);
			$query = "SELECT `{$campoId}`, `{$campoPeso}` "
				."FROM `{$table}`"
				."WHERE `{$campoPadre}` = '{$row[$campoPadre]}' "
				."AND `{$campoPeso}` <= '{$row[$campoPeso]}' "
				."ORDER BY `{$campoPeso}`, `{$campoId}` ";
		}
		else
		{
			$row = $db->GetRow("SELECT `{$campoPeso}` FROM `{$table}` WHERE `{$campoId}` = '{$id}'", true, false);
			$query = "SELECT `{$campoId}`, `{$campoPeso}` "
				."FROM `{$table}`"
				."WHERE `{$campoPeso}` <= '{$row[$campoPeso]}' "
				."ORDER BY `{$campoPeso}`, `{$campoId}` ";
		}

		$rows = $db->GetRows($query);
		$size = sizeof($rows);
		if($size > 1)
		{
			for($c=0; $c<$size; $c++)
			{
				if($rows[$c][$campoId] == $id)
				{
					$rows[$c][$campoPeso] = isset($rows[$c-1]) ? $rows[$c-1][$campoPeso] - 1 : -1;
					$db->Execute("UPDATE `{$table}` SET `{$campoPeso}` = '{$rows[$c][$campoPeso]}' WHERE `{$campoId}` = '{$id}'");
					break;
				}
			}
		}
	}
	/**
	 * Baja un registro por peso
	 *
	 * @param int $id
	 * @param string $table
	 * @param string $campoPadre
	 * @param string $campoPeso
	 * @param string $campoId
	 */
	static function _BajarRegistro($id, $table, $campoPadre='padre_id', $campoPeso='peso', $campoId='id')
	{
		global $db;

		if($campoPadre)
		{
			$row = $db->GetRow("SELECT `{$campoPeso}`, `{$campoPadre}` FROM `{$table}` WHERE `{$campoId}` = '{$id}'", true, false);
			$query = "SELECT `{$campoId}`, `{$campoPeso}` "
				."FROM `{$table}`"
				."WHERE `{$campoPadre}` = '{$row[$campoPadre]}' "
				."AND `{$campoPeso}` >= '{$row[$campoPeso]}' "
				."ORDER BY `{$campoPeso}`, `{$campoId}` "
				."LIMIT 2";
		}
		else
		{
			$row = $db->GetRow("SELECT `{$campoPeso}` FROM `{$table}` WHERE `{$campoId}` = '{$id}'", true, false);
			$query = "SELECT `{$campoId}`, `{$campoPeso}` "
				."FROM `{$table}`"
				."WHERE `{$campoPeso}` >= '{$row[$campoPeso]}' "
				."ORDER BY `{$campoPeso}`, `{$campoId}` "
				."LIMIT 2";
		}

		$rows = $db->GetRows($query);
		$size = sizeof($rows);
		if($size > 1)
		{
			for($c=0; $c<$size; $c++)
			{
				if($rows[$c][$campoId] == $id)
				{
					$rows[$c][$campoPeso] = isset($rows[$c+1]) ? $rows[$c+1][$campoPeso] + 1 : 1;
					$db->Execute("UPDATE `{$table}` SET `{$campoPeso}` = '{$rows[$c][$campoPeso]}' WHERE `{$campoId}` = '{$id}'");
					break;
				}
			}
		}
	}
	/**
	 * Obtiene un array con los perfiles, donde los índices son los Id. y los valores los nombres.
	 *
	 * @return array
	 */
	static function _GetPerfiles($min=1, $max=999999, $orderby='pi.nombre ASC')
	{
		global $db;

		return $db->GetArrayForHTMLSelectQuery("SELECT p.id, IF(pi.nombre != '' AND pi.nombre IS NOT null, pi.nombre, '[perfil]') AS nombre FROM ".SWPHP_DBPREFIX."perfiles AS p LEFT JOIN ".SWPHP_DBPREFIX."perfiles_idiomas AS pi ON pi.perfil_id = p.id AND pi.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}' WHERE p.id >= {$min} AND p.id <= {$max} ORDER BY {$orderby}");
	}
	/**
	 * Obtiene un array con los perfiles.
	 *
	 * @return array
	 */
	static function _GetPerfilesRows($min=1, $max=999999, $orderby='pi.nombre ASC')
	{
		global $db;

		return $db->GetRows("SELECT p.id, IF(pi.nombre != '' AND pi.nombre IS NOT null, pi.nombre, '[perfil]') AS nombre FROM ".SWPHP_DBPREFIX."perfiles AS p LEFT JOIN ".SWPHP_DBPREFIX."perfiles_idiomas AS pi ON pi.perfil_id = p.id AND pi.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}' WHERE p.id >= {$min} AND p.id <= {$max} ORDER BY {$orderby}");
	}
	/**
	 * Obtiene un array con los perfiles, donde los índices son los Id. y los valores los nombres.
	 *
	 * @param array $ids
	 * @return array
	 */
	static function _GetPerfilesIn($ids, $orderby='pi.nombre ASC')
	{
		global $db;

		if(is_array($ids))
		{
			$ids = implode(',', $ids);
		}

		return empty($ids) ? array() : $db->GetArrayForHTMLSelectQuery("SELECT p.id, IF(pi.nombre != '' AND pi.nombre IS NOT null, pi.nombre, '[perfil]') AS nombre FROM ".SWPHP_DBPREFIX."perfiles AS p LEFT JOIN ".SWPHP_DBPREFIX."perfiles_idiomas AS pi ON pi.perfil_id = p.id AND pi.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}' WHERE p.id IN({$ids}) ORDER BY {$orderby}");
	}
	/**
	 * Obtiene los datos de un perfil
	 *
	 * @param int $id
	 * @return array
	 *
	static function _GetPerfil($id)
	{
		global $db;

		return $db->GetRow("SELECT p.id, IF(pi.nombre != '' AND pi.nombre IS NOT null, pi.nombre, '[perfil]') AS nombre FROM ".SWPHP_DBPREFIX."perfiles AS p LEFT JOIN ".SWPHP_DBPREFIX."perfiles_idiomas AS pi ON pi.perfil_id = p.id AND pi.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}' WHERE p.id = {$id}");
	}*/
	/**
	 * Obtiene un array con los camposuser
	 *
	 * @return array
	 */
	static function _GetCamposUser($perfil_id=null)
	{
		global $db;

		$filtro = is_null($perfil_id) ? '1' : "perfil_id = '{$perfil_id}'";

		$rows = array();
		$rs = DB_ResultSet::Factory($db->GetExtension());
		$rs->Set = $db->Execute("SELECT c.id, c.permiso_id, ci.nombre, c.info FROM ".SWPHP_DBPREFIX."camposuser AS c JOIN ".SWPHP_DBPREFIX."camposuser_idiomas AS ci ON c.id = ci.campouser_id AND ci.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}' WHERE c.id IN(SELECT campouser_id FROM ".SWPHP_DBPREFIX."camposuser_perfiles WHERE {$filtro}) ORDER BY c.peso ASC, c.id ASC");
		while(($reg = $rs->FetchAssoc()))
		{
			$reg['perfiles'] = $db->GetValuesQuery("SELECT perfil_id FROM ".SWPHP_DBPREFIX."camposuser_perfiles WHERE campouser_id = '{$reg['id']}'", false, true);
			$reg['permisos'] = $db->GetArrayForHTMLSelectQuery("SELECT perfil_id, permiso_id FROM ".SWPHP_DBPREFIX."camposuser_perfiles WHERE campouser_id = '{$reg['id']}'", null, false, true);
			$rows[$reg['id']] = $reg;
		}

		return $rows;
	}
	/**
	 * Obtiene el id de permiso para un módulo y un perfil de usuario
	 *
	 * @param int $modulo_id
	 * @param int $perfil_id
	 * @return int
	 */
	static function _GetPermiso($modulo_id, $perfil_id=null)
	{
		global $db;

		if(is_null($perfil_id))
		{
			$perfil_id = isset($_SESSION['user']['autenticado']) && $_SESSION['user']['autenticado'] ? $_SESSION['user']['perfil_id'] : 0;
		}

		$permiso_id = $db->GetValue('permiso_id', SWPHP_DBPREFIX."modulos_perfiles", "WHERE modulo_id = '{$modulo_id}' AND perfil_id = '{$perfil_id}'");

		return intval($permiso_id);
	}
	/**
	 * Obtiene el conjunto de módulos que existen para un entorno (frontend|backend|ambos), un perfil y un nivel de permiso dados
	 *
	 * @param string $entorno
	 * @param int $perfil_id
	 * @param int $permiso_id
	 * @param bool $soloActivos
	 * @param array $modulos
	 * @return array
	 */
	static function _GetModulos($entorno='', $perfil_id=null, $permiso_id=SWPHP_PERMISO_ACCESOPROHIBIDO, $soloActivos=false, $modulos=array(), $ordenar=true)
	{
		global $db;

		$filtroactivo = $soloActivos ? 'm.activo = 1' : '1';
		$filtroperfil = is_null($perfil_id) ? '1' : "mp.perfil_id = '{$perfil_id}'";
		$filtroClave = empty($modulos) ? '1' : sprintf("clave IN('%s')", implode("','", $modulos));

		$query = "SELECT DISTINCT m.id, m.clave, m.titulo, m.descripcion, m.icono, ".($perfil_id ? 'mp.permiso_id' : SWPHP_PERMISO_ACCESOPROHIBIDO.' AS permiso_id').", m.version"
			." FROM ".SWPHP_DBPREFIX."modulos AS m"
			." JOIN ".SWPHP_DBPREFIX."modulos_perfiles AS mp ON (mp.modulo_id = m.id AND {$filtroperfil} AND mp.permiso_id >= '{$permiso_id}')"
			." WHERE {$filtroactivo} AND {$filtroClave}";
		if($ordenar)
		{
			$query .= " ORDER BY m.titulo";
		}

		$rows = $db->GetRows($query);

		//Quito los que no se encuentran o no son gestionables
		for($c=0,$size=sizeof($rows); $c<$size; $c++)
		{
			switch($entorno)
			{
				case 'frontend':
				case 'publico':
				case 'publica':
				case 'index':
					if(!is_file(SWPHP_MODULES_DIR.$rows[$c]['clave'].'/index.php'))
					{
						unset($rows[$c]);
					}
					break;
				case 'backend':
				case 'admin':
				case 'panel':
					if(!is_file(SWPHP_MODULES_DIR.$rows[$c]['clave'].'/admin/index.php'))
					{
						unset($rows[$c]);
					}
					break;
				default:
					if(!is_dir(SWPHP_MODULES_DIR.$rows[$c]['clave']))
					{
						unset($rows[$c]);
					}
					break;
			}
		}

		if(!$ordenar && !empty($modulos))
		{
			$aux = array();

			foreach($modulos as $clave)
			{
				foreach($rows as $row)
				{
					if($row['clave'] == $clave)
					{
						$aux[] = $row;
						break;
					}
				}
			}

			$rows = $aux;
		}

		return $rows;
	}
	/**
	 * Obtiene el conjunto de módulos activos del panel de administración para un perfil y un nivel de permiso dados
	 *
	 * @param int $perfil_id
	 * @param int $permiso_id
	 * @return array
	 */
	static function _GetModulosAdmin($perfil_id=0, $permiso_id=SWPHP_PERMISO_LECTURAYESCRITURA)
	{
		return SWPHP::_GetModulos('backend', $perfil_id, $permiso_id, true);
	}
	/**
	 * Establece el idioma actual (en sesión)
	 *
	 * @param mixed $idioma
	 * @return bool
	 */
	static function _SetIdiomaActual($idioma=null)
	{
		global $swphp_idiomas;

		if(is_null($idioma))
		{
			if(isset($_GET['lang']))
			{
				$idioma = $_GET['lang'];
			}
			elseif(isset($_SESSION['SWPHP_IDIOMA_ACTUAL'], $swphp_idiomas[$_SESSION['SWPHP_IDIOMA_ACTUAL']]))
			{
				SWPHP::_SetLocale($swphp_idiomas[$_SESSION['SWPHP_IDIOMA_ACTUAL']]['codigolang']);

				return true;
			}
			else
			{
				$idioma = SWPHP_IDIOMA_POR_DEFECTO;
			}
		}

		foreach($swphp_idiomas as $key=>$value)
		{
			if($key == $idioma || $value['codigoiso'] == $idioma)
			{
				$_SESSION['SWPHP_IDIOMA_ACTUAL'] = $key;
				$_SESSION['SWPHP_IDIOMA_ACTUAL_CODIGOISO'] = $value['codigoiso'];

				SWPHP::_SetLocale($value['codigolang']);

				return true;
			}
		}

		if(!isset($_SESSION['SWPHP_IDIOMA_ACTUAL']))
		{
			$_SESSION['SWPHP_IDIOMA_ACTUAL'] = SWPHP_IDIOMA_POR_DEFECTO;
			$_SESSION['SWPHP_IDIOMA_ACTUAL_CODIGOISO'] = SWPHP_IDIOMA_POR_DEFECTO_CODIGOISO;
		}

		SWPHP::_SetLocale($swphp_idiomas[$_SESSION['SWPHP_IDIOMA_ACTUAL']]['codigolang']);

		return false;
	}
	/**
	 * Establece la localización (locale) según un código del tipo xx_XX
	 *
	 * @param string $codigolang
	 */
	static function _SetLocale($codigolang)
	{
		setlocale(LC_NUMERIC, 'ENG', 'en_EN', 'en_EN.'.SWPHP_CHARSET, 'en_EN.ISO8859-1', 'en_EN.ISO-8859-15', 'en_EN@euro');

		switch($codigolang)
		{
			case 'es_ES':
				$aux = setlocale(LC_TIME, 'ESP', 'es_ES', 'es_ES.'.SWPHP_CHARSET, 'es_ES.ISO8859-1', 'es_ES.ISO-8859-15', 'es_ES@euro');
				break;
			case 'en_EN':
				$aux = setlocale(LC_TIME, 'ENG', 'en_EN', 'en_EN.'.SWPHP_CHARSET, 'en_EN.ISO8859-1', 'en_EN.ISO-8859-15', 'en_EN@euro');
				break;
			case 'fr_FR':
				$aux = setlocale(LC_TIME, 'FRA', 'fr_FR', 'fr_FR.'.SWPHP_CHARSET, 'fr_FR.ISO8859-1', 'fr_FR.ISO-8859-15', 'fr_FR@euro');
				break;
			case 'de_DE':
				$aux = setlocale(LC_TIME, 'DEU', 'de_DE', 'de_DE.'.SWPHP_CHARSET, 'de_DE.ISO8859-1', 'de_DE.ISO-8859-15', 'de_DE@euro');
				break;
			case 'it_IT':
				$aux = setlocale(LC_TIME, 'ITA', 'it_IT', 'it_IT.'.SWPHP_CHARSET, 'it_IT.ISO8859-1', 'it_IT.ISO-8859-15', 'it_IT@euro');
				break;
			case 'ca_ES':
				$aux = setlocale(LC_TIME, 'CAT', 'ca_ES', 'ca_ES.'.SWPHP_CHARSET, 'ca_ES.ISO8859-1', 'ca_ES.ISO-8859-15', 'ca_ES@euro');
				break;
			default:
				$aux = setlocale(LC_TIME, $codigolang, $codigolang.'.'.SWPHP_CHARSET, $codigolang.'.ISO8859-1', $codigolang.'.ISO-8859-15', $codigolang.'.@euro');
				break;
		}

		setlocale(LC_MONETARY, $aux);
	}
	/**
	 * Traduce
	 *
	 * @param string $str
	 * @param string $concepto
	 * @param string $isocode
	 * @return string
	 */
	static function _L($str, $concepto, $isocode=null)
	{
		return l($str, $concepto, $isocode);
	}
	/**
	 * Obtiene un array con los nombres de los themes presentes
	 * (los nombres de los directorios correspondientes)
	 * Los índices del array también son esos nombres,
	 * así sirve para asignarlo a html select.
	 *
	 * @return array
	 */
	static function _GetThemes()
	{
		return SWPHP::_GetDirsForHTMLSelect(SWPHP_THEMES_DIR);
	}
	/**
	 * Obtiene un array asociativo con los subdirectorios encontrados en un directorio.
	 * Los índices son los mismos que los valores: los nombres de los subdirectorios.
	 *
	 * @param string $dir
	 * @param bool $includepath
	 * @return array
	 */
	static function _GetDirsForHTMLSelect($dir, $includepath=false)
	{
		$arr = array();
		$dirs = getDirs($dir, $includepath);
        sort($dirs);
        foreach($dirs as $item)
		{
			$arr[$item] = $item;
		}

		return $arr;
	}
	/**
	 * Obtiene un array asociativo con los archivos encontrados en un directorio.
	 * Los índices son los mismos que los valores: los nombres de los archivos.
	 * Permite indicar las extensiones excluidas.
	 *
	 * @param string $dir
	 * @param array $excludedextensions
	 * @param bool $includepath
	 * @return array
	 */
	static function _GetFilesForHTMLSelect($dir, $excludedextensions=array(), $includepath=false)
	{
		$arr = array();
		$files = getFiles($dir, $excludedextensions, $includepath, false);
		foreach($files as $file)
		{
			$arr[$file] = $file;
		}

		return $arr;
	}
	/**
	 * Obtiene un array asociativo con los archivos encontrados en un directorio.
	 * Los índices son los mismos que los valores: los nombres de los archivos.
	 * Permite indicar las extensiones permitidas.
	 *
	 * @param string $dir
	 * @param array $includedextensions
	 * @param bool $includepath
	 * @return array
	 */
	static function _GetFilesExtForHTMLSelect($dir, $includedextensions=array(), $includepath=false)
	{
		$arr = array();
		$files = getFilesExt($dir, $includedextensions, $includepath, false);
		foreach($files as $file)
		{
			$arr[$file] = $file;
		}

		return $arr;
	}
	/**
	 * Genera el archivo .htaccess para el raiz de este Simple web PHP
	 *
	 * @return bool
	 */
	static function _GenerarHtaccess($return=false)
	{
		global $db, $swphp_config;

		//EMPEZAMOS
		$htaccess = sprintf('#
# Jose Carlos Cruz Parra
# programadorphpfreelance.com
# %s
#
# SIMPLE WEB PHP EDICION ADMINISTRADA
#
# .htaccess
#

# Este archivo ha sido autogenerado por SWPHP EA

SetEnvIfNoCase User-Agent ".*spider" bad_bot

<Limit GET POST>
order allow,deny
allow from all
Deny from env=bad_bot
</Limit>

<Files ".htaccess">
deny from all
</Files>

# php_flag register_globals off
# php_flag magic_quotes_gpc off
',
			date('Y-m-d')
			);

		//ERRORES DEL SERVIDOR
		$errores = array(
			400, 401, 402, 403, 404, 405, 406,
			408,
			410, 411, 412, 413, 414, 415,
			500, 501
			);
		foreach($errores as $error)
		{
			$htaccess .= sprintf("\nErrorDocument %u /%sservererror.php?error=%u", $error, $swphp_config['path'], $error);
		}

		//CADUCIDAD DE IMAGENES, JS, CSS
		$htaccess .= '

# Añadir una fecha de caducidad a las imágenes, javascript, css y texto
ExpiresActive On
ExpiresByType image/gif A2592000
ExpiresByType image/png A2592000
ExpiresByType image/jpg A2592000
ExpiresByType image/jpeg A2592000
ExpiresByType text/javascript A2592000
ExpiresByType application/javascript A2592000
ExpiresByType application/x-javascript A2592000
ExpiresByType text/css A2592000
';

		//COMPRIMIR SALIDA
		$htaccess .= '
# Comprimir mediante el método deflate
AddOutputFilterByType DEFLATE text/plain
AddOutputFilterByType DEFLATE text/html
AddOutputFilterByType DEFLATE text/xml
AddOutputFilterByType DEFLATE text/css
AddOutputFilterByType DEFLATE text/javascript
AddOutputFilterByType DEFLATE application/xml
AddOutputFilterByType DEFLATE application/xhtml+xml
AddOutputFilterByType DEFLATE application/rss+xml
AddOutputFilterByType DEFLATE application/javascript
AddOutputFilterByType DEFLATE application/x-javascript
AddOutputFilterByType DEFLATE application/x-httpd-php
AddOutputFilterByType DEFLATE application/x-httpd-fastphp
AddOutputFilterByType DEFLATE image/svg+xml

# <IfModule mod_gzip.c>
# mod_gzip_on Yes
# mod_gzip_item_include file .html$
# mod_gzip_item_include file .php$
# mod_gzip_item_include file .css$
# mod_gzip_item_include file .js$
# mod_gzip_item_include mime ^application/javascript$
# mod_gzip_item_include mime ^application/x-javascript$
# mod_gzip_item_include mime ^text/.*
# mod_gzip_item_include handler ^application/x-httpd-php
# mod_gzip_item_exclude mime ^image/.*
# </IfModule>
';

		//DIRECTORIO DE ADMINISTRACIÓN, Y MÁS
		$admindir = quitarBarra(SWPHP_ADMIN_DIR);
		$htaccess .= sprintf('
RewriteEngine On

# Bloquear agente libwww-perl, ya que muchos atacantes lo usan
RewriteCond %%{HTTP_USER_AGENT} libwww-perl.*
RewriteRule .* ? [F,L]

# General js
# RewriteRule ^js/global.js$ js/global.js.php [L]

# General css
RewriteRule ^themes/([^/.]+)/css/global.css$ themes/$1/css/global.css.php [L]
RewriteRule ^themes/([^/.]+)/css/global_mobile.css$ themes/$1/css/global.css.php?mobile=1 [L]

RewriteRule ^(img|css|js|files|cron|utils|install|zbblock|awstats|repositorio)/.*$ - [PT]

# Backend
RewriteRule ^(%s)/?$ $1/index.php [L]
RewriteRule ^(%s)/([^/.]+)/?$ $1/index.php?mod=$2&%%{QUERY_STRING} [L]
RewriteRule ^(%s)/([^/.]+)/([^/.]+)/?$ $1/index.php?mod=$2&sub=$3&%%{QUERY_STRING} [L]
RewriteRule ^(%s)/([^/.]+)/([^/.]+)/([^/.]+)/?$ $1/index.php?mod=$2&sub=$3&ssub=$4&%%{QUERY_STRING} [L]

# Idioma
RewriteRule ^([a-z][a-z])/(.*)/?$ $2/?lang=$1&%%{QUERY_STRING} [L]

# Especiales
RewriteRule ^(logout|solicitarclave)/?$ $1.php [L]
',
			$admindir, $admindir, $admindir, $admindir
			);

		//REVISAR CADA MÓDULO (buscar htaccess en info.php)
		$claves = $db->GetValuesQuery("SELECT clave FROM ".SWPHP_DBPREFIX."modulos", false);
		$ultimo = '';
		foreach($claves as $clave)
		{
			$file = SWPHP_MODULES_DIR."{$clave}/info.php";
			if(is_file($file))
			{
				include($file);

				if(isset($modulo['htaccess']) && $modulo['htaccess'] && $modulo['htaccess'] != $ultimo)
				{
					//TODO: ¿poner la url amigable de la pagina en vez del modulo?
					//¿forzar la friendly de la página al módulo?

					$ultimo = $modulo['htaccess'];
					$htaccess .= "\n# Módulo {$clave}\n{$modulo['htaccess']}\n";
				}
			}
		}

		//TERMINAMOS
		$htaccess .= '
# Frontend
RewriteRule ^([^/.]+)/?$ index.php?page0=$1&%{QUERY_STRING} [L]
RewriteRule ^([^/.]+)/([^/.]+)/?$ index.php?page0=$1&page1=$2&%{QUERY_STRING} [L]
RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/?$ index.php?page0=$1&page1=$2&page2=$3&%{QUERY_STRING} [L]
RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/?$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&%{QUERY_STRING} [L]
RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/?$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&page4=$5&%{QUERY_STRING} [L]
RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/?$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&page4=$5&page5=$6&%{QUERY_STRING} [L]
RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/?$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&page4=$5&page5=$6&page6=$7&%{QUERY_STRING} [L]
RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/?$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&page4=$5&page5=$6&page6=$7&page7=$8&%{QUERY_STRING} [L]
RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/?$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&page4=$5&page5=$6&page6=$7&page7=$8&page8=$9&%{QUERY_STRING} [L]
# //TODO: ¿habilitar esto?
# RewriteRule ^([^/.]+)/index.php$ index.php?page0=$1&%{QUERY_STRING} [L]
# RewriteRule ^([^/.]+)/([^/.]+)/index.php$ index.php?page0=$1&page1=$2&%{QUERY_STRING} [L]
# RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/index.php$ index.php?page0=$1&page1=$2&page2=$3&%{QUERY_STRING} [L]
# RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/index.php$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&%{QUERY_STRING} [L]
# RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/index.php$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&page4=$5&%{QUERY_STRING} [L]
# RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/index.php$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&page4=$5&page5=$6&%{QUERY_STRING} [L]
# RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/index.php$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&page4=$5&page5=$6&page6=$7&%{QUERY_STRING} [L]
# RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/index.php$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&page4=$5&page5=$6&page6=$7&page7=$8&%{QUERY_STRING} [L]
# RewriteRule ^([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/([^/.]+)/index.php$ index.php?page0=$1&page1=$2&page2=$3&page3=$4&page4=$5&page5=$6&page6=$7&page7=$8&page8=$9&%{QUERY_STRING} [L]
';

		if($return)
		{
			return $htaccess;
		}

		//TODO: escribir archivo y devolver si se ha podido o no

		return false;
	}
	/**
	 * Comprueba la versión de SWPHP y si es necesario actualiza
	 *
	 * @param string $versionIns
	 */
	static function _CheckVersionSWPHP($versionIns=null)
	{
		global $db, $swphp_config, $swphp_idiomas;

		if(is_null($versionIns))
		{
			$versionIns = SWPHP::_GetConfigValue('version', 'usointerno', false);
		}

		if(SWPHP_VERSION > $versionIns)
		{
			SWPHP::_Msg("Nueva versión de SWPHP EA detectada: <b>".SWPHP_VERSION."</b>", 'aviso');
			SWPHP::_Msg("Actualización de SWPHP EA iniciada", 'info');

			$dir = SWPHP_INC_DIR.'upd/';

			if(is_dir($dir))
			{
				$files = getFilesExt($dir, array('php'));
				sort($files);

				$updOk = true;
				foreach($files as $file)
				{
					$versionFile = getName($file);
					if($versionFile > $versionIns && $versionFile <= SWPHP_VERSION)
					{
						$updOk = false;

						include($dir.$file);

						if(!$updOk)
						{
							break;
						}
					}
				}

				if($updOk && SWPHP::_CreateConfigValue('version', SWPHP_VERSION, 'usointerno', 'tag=input;type=text;size=10;maxlength=12', 'Versión de SWPHP EA', 3, true))
				{
					//Actualizar módulos instalados y activos
					$modulos = SWPHP::_GetModulos('', null, SWPHP_PERMISO_ACCESOPROHIBIDO, true);
					foreach($modulos as $modulo)
					{
						SWPHP::_CheckVersionModulo($modulo['clave'], $modulo['version']);
					}
				}
				else
				{
					SWPHP::_Msg("No se pudo completar la actualización de SWPHP EA, revísalo a mano o contacta con un experto en <a class=\"external\" href=\"http://simplewebphp.com\">simplewebphp.com</a>", 'error');

					return false;
				}
			}
			else
			{
				SWPHP::_Msg("No existe el directorio de archivos de actualización", 'aviso');
			}

			//Eliminar tpl compilados para que se regeneren
			$files = getFilesExt('templates_c', array('php'), true);
			foreach($files as $file)
			{
				unlink($file);
			}

			SWPHP::_Msg("Actualización de SWPHP EA terminada", 'check');
		}

		return true;
	}
	/**
	 * Comprueba la versión de un módulo y si es necesario lo actualiza
	 *
	 * @param string $clave
	 * @param string $versionIns
	 */
	static function _CheckVersionModulo($clave, $versionIns=null)
	{
		global $db, $swphp_config;

		if(is_null($versionIns))
		{
			$versionIns = $db->GetValueQuery("SELECT version FROM ".SWPHP_DBPREFIX."modulos WHERE clave = '{$clave}'", false);
		}

		$info = SWPHP::_GetModuloInfo($clave);

		if($info['version'] > $versionIns)
		{
			SWPHP::_Msg("Nueva versión del módulo <b>{$clave}</b> detectada: <b>{$info['version']}</b>", 'aviso');

			//Comprobar que la versión de SWPHP es soportada por el módulo
			if(isset($info['swphp']) && SWPHP_VERSION >= $info['swphp'])
			{
				SWPHP::_Msg("Actualización del módulo <b>{$clave}</b> iniciada", 'info');

				$updOk = true;
				$dir = SWPHP_MODULES_DIR.$clave.'/upd/';

				if(is_dir($dir))
				{
					$files = getFilesExt($dir, array('php'));
					sort($files);

					foreach($files as $file)
					{
						$versionFile = getName($file);
						if($versionFile > $versionIns && $versionFile <= $info['version'])
						{
							$updOk = false;

							include($dir.$file);

							if(!$updOk)
							{
								break;
							}
						}
					}
				}
				else
				{
					SWPHP::_Msg("No existe el directorio de archivos de actualización", 'aviso');
				}

				if(!$updOk || !$db->Execute("UPDATE ".SWPHP_DBPREFIX."modulos SET version = '{$info['version']}' WHERE clave = '{$clave}'"))
				{
					SWPHP::_Msg("No se pudo completar la actualización del módulo <b>{$clave}</b>, revísalo a mano o contacta con un experto en <a class=\"external\" href=\"http://simplewebphp.com\">simplewebphp.com</a>", 'error');

					return false;
				}

				SWPHP::_Msg("Actualización del módulo <b>{$clave}</b> terminada", 'check');
			}
			else
			{
				SWPHP::_Msg("No se puede realizar la actualización, <b>{$info['clave']} {$info['version']}</b> es para la versión de SWPHP <b>{$info['swphp']}</b> o superior, pero tu versión es <b>".SWPHP_VERSION."</b>.<br />Tienes que actualizar tu sistema SWPHP para poder actualizar el módulo.<br />Si necesitas ayuda, puedes contactar con un experto en <a class=\"external\" href=\"http://simplewebphp.com\">simplewebphp.com</a>", "aviso");
			}
		}

		return true;
	}
	/**
	 * Obtiene un array asociativo con la información correspondiente a un módulo,
	 * proporcionada desde el info.php de ese módulo y completada
	 * con los valores por defecto para los índices que no estén definidos.
	 *
	 * @param string $clave
	 * @return array
	 */
	static function _GetModuloInfo($clave)
	{
		//Trato de coger la información proporcionada por el módulo
		$infopath = SWPHP_MODULES_DIR."{$clave}/info.php";
		if(is_file($infopath))
		{
			include($infopath);
		}

		//Si no he encontrado la información
		if(!isset($modulo))
		{
			$modulo = array();
		}

		//Completo la información que pueda faltar
		$defaults = array(
			'clave'=>$clave,
			'titulo'=>ucfirst($clave),
			'descripcion'=>'',
			'icono'=>'blank.png',
			'core'=>'0',
			'autor'=>'',
			'url'=>'',
			'version'=>'',
			'notas'=>'',
			);
		foreach($defaults as $key=>$value)
		{
			if(!isset($modulo[$key]))
			{
				$modulo[$key] = $value;
			}
		}

		return $modulo;
	}
	/**
	 * Añade texto a $swphp_extrahead para incluirlo en el html head
	 *
	 * @param string $str
	 */
	static function _AddExtrahead($str)
	{
		global $swphp_extrahead;

		$swphp_extrahead .= "{$str}\n";
	}
	/**
	 * Añade texto a $swphp_documentready para incluirlo en el html head
	 *
	 * @param string $str
	 */
	static function _AddDocumentReady($str)
	{
		global $swphp_documentready;

		$swphp_documentready .= "{$str}\n";
	}
	/**
	 * Transforma una cadena en meta description
	 *
	 * @param string $str
	 * @param bool $html_entity_decode
	 * @param int $maxlen
	 * @param int $desviacion
	 * @param array $palabritas
	 * @return string
	 */
	static function _Str2MetaDescription($str, $html_entity_decode=false, $maxlen=180, $desviacion=20, $palabritas=array('a', 'de', 'por', 'para', 'y', 'sin', 'desde', 'con', 'e', 'o', 'ó', 'sus'))
	{
		return string2description($str, $html_entity_decode, $maxlen, $desviacion, $palabritas);
	}
	/**
	 * Transforma una cadena en meta keywords
	 *
	 * @param string $str
	 * @param bool $html_entity_decode
	 * @param int $maxwords
	 * @param int $minlength
	 * @return string
	 */
	static function _Str2MetaKeywords($str, $html_entity_decode=false, $maxwords=15, $minlength=3)
	{
		return string2keywords($str, $html_entity_decode, $maxwords, $minlength);
	}
	/**
	 * Para mostrar el menú de submódulos.
	 * Internamente asigna el valor para $admin_template, así que no hay que hacerlo desde el módulo.
	 *
	 * @param string $mod_clave
	 * @param string $permiso_id
	 * @param string $sub
	 */
	static function _MenuSubmodulos($mod_clave=null, $permiso_id=null, $sub=null)
	{
		global $admin_template, $smarty;

		if(empty($mod_clave))
		{
			global $swphp_mod_clave;

			$mod_clave = $swphp_mod_clave;
		}

		if(is_null($permiso_id))
		{
			global $swphp_permiso_id;

			$permiso_id = $swphp_permiso_id;
		}

		if(is_null($sub))
		{
			global $swphp_sub;

			$sub = $swphp_sub;
		}

		$smarty->assign('swphp_modulos', SWPHP::_GetSubmodulos($permiso_id, $mod_clave, $sub));

		$admin_template = 'admin/default.tpl';
	}
	/**
	 * Codifica una cadena usando MyCoder::Encode y SWPHP_FRASE_SECRETA
	 *
	 * @param string $str
	 * @return string
	 */
	static function _Encode($str)
	{
		include_once(SWPHP_COMMON_DIR.'classes/MyCoder.class.php');

		return MyCoder::Encode($str, SWPHP_FRASE_SECRETA);
	}
	/**
	 * Decodifica una cadena usando MyCoder::Encode y SWPHP_FRASE_SECRETA
	 *
	 * @param string $str
	 * @return string
	 */
	static function _Decode($str)
	{
		include_once(SWPHP_COMMON_DIR.'classes/MyCoder.class.php');

		return MyCoder::Decode($str, SWPHP_FRASE_SECRETA);
	}
	/**
	 * Ejecuta un conjunto de sentencias SQL
	 * donde el prefijo de las tablas aparece como swphp_ y es reemplazado por el de la instalación actual
	 *
	 * @param string $sql
	 * @param bool $droptable
	 * @return bool
	 */
	static function _ExecuteSQL($sql, $droptable=false, $mostrarMsg=false)
	{
	    global $db;

	    $sql = trim($sql);

	    if($sql && $sql[strlen($sql)-1] != ";")
	    {
	    	$sql .= ';';
	    }

	    $ok = true;
	    $query = "";
		$lines = explode("\n", $sql);
		$c = 0;
		while(isset($lines[$c]) && $c < 100000)
		{
			$str = $lines[$c];
			$c++;
			if($str != '' && substr($str, 0, 1) != "#" && substr($str, 0, 3) !== '-- ')
			{
				$query .= $str;
			}
			$aux = rtrim($str);
			if($aux && $aux[strlen($aux)-1] == ";")
			{
				$query = trim(str_replace(
					array("DROP TABLE IF EXISTS `swphp_", "CREATE TABLE IF NOT EXISTS `swphp_", "INSERT INTO `swphp_", "UPDATE `swphp_", "ALTER TABLE `swphp_"),
					array("DROP TABLE IF EXISTS `".SWPHP_DBPREFIX, "CREATE TABLE IF NOT EXISTS `".SWPHP_DBPREFIX, "INSERT INTO `".SWPHP_DBPREFIX, "UPDATE `".SWPHP_DBPREFIX, "ALTER TABLE `".SWPHP_DBPREFIX),
					$query
					));
				if($droptable || strtoupper(substr($query, 0, 11)) != "DROP TABLE ")
				{
	                if($mostrarMsg)
	                {
	                	SWPHP::_Msg($query);
	                }

					if($db->Execute($query))
					{
						if($mostrarMsg)
						{
							SWPHP::_Msg('OK, '.$db->AffectedRows().' filas afectadas', 'check');
						}
					}
					else
					{
						if($mostrarMsg)
						{
							SWPHP::_Msg('ERROR, '.$db->Error(), 'error');
						}
						else
						{
							SWPHP::_Msg('Error ejecutando sentencia SQL<br />'.SWPHP::_HtmlEntities($query).'<br />'.$db->Error(), 'error');
						}

	                    $ok = false;
					}
				}
				$query = "";
			}
		}

	    return $ok;
	}
	/**
	 * Ejecuta un conjunto de sentencias SQL guardado en un archivo
	 * donde el prefijo de las tablas aparece como swphp_ y es reemplazado por el de la instalación actual
	 *
	 * @param string $filepath
	 * @param bool $droptable
	 * @return bool
	 */
	static function _ExecuteSQLFile($filepath, $droptable=false)
	{
		$sql = file_get_contents($filepath);

	    return $sql === false ? false : SWPHP::_ExecuteSQL($sql, $droptable);
	}
	/**
	 * Obtiene el icono (nombre de archivo) correspondiente a una extensión de archivo
	 *
	 * @param string $extension
	 * @return string
	 */
	static function _Ext2Ico($extension)
	{
		switch($extension)
		{
			case 'jpeg':
				$extension = 'jpg';
				break;
			case 'docx':
			case 'pptx':
				$extension = substr($extension, 0 -1);
				break;
			default:
				//nada
				break;
		}

		return $extension.'.png';
	}
	/**
	 * Obtiene el icono (ruta) correspondiente a un elemento
	 *
	 * @param string $tipo
	 * @param string $aux
	 * @param int $width
	 * @return string
	 */
	static function _IconFile($tipo, $aux='', $width=48)
	{
		switch($tipo)
		{
			case 'carpeta':
			case 'folder':
			case 'dir':
				$icon = $aux ? 'folder_full.png' : 'folder_empty.png';
				break;
			case 'archivo':
			case 'file':
				$icon = SWPHP::_Ext2Ico($aux);
			default:
				break;
		}

		$path = SWPHP_THEME_DIR.'img/files/'.$width.'x'.$width.'/'.$icon;

		return is_file($path) ? $path : SWPHP_THEME_DIR.'img/files/'.$width.'x'.$width.'/_blank.png';
	}
	/**
	 * Obtiene un array a partir de módulos para utilizarlo como menú
	 *
	 * @param array $modulos
	 * @param int $perfil_id
	 * @param int $permiso_id
	 * @param bool $soloActivos
	 * @return array
	 */
	static function _Modulos2Menu($modulos=array(), $perfil_id=null, $permiso_id=SWPHP_PERMISO_ACCESOPROHIBIDO, $soloActivos=true, $ordenar=false)
	{
		global $db;

		if(is_null($perfil_id))
		{
			$perfil_id = isset($_SESSION['user']['autenticado']) && $_SESSION['user']['autenticado'] ? $_SESSION['user']['perfil_id'] : 0;
		}

		$menu = array();
		$modulos = SWPHP::_GetModulos('admin', $perfil_id, $permiso_id, $soloActivos, $modulos, $ordenar);
		foreach($modulos as $modulo)
		{
			$modulo['hijos'] = array();

			$submodulos = SWPHP::_GetSubmodulos($modulo['permiso_id'], $modulo['clave']);
			foreach($submodulos as $submodulo)
			{
				$submodulo['hijos'] = array();

				$subsubmodulos = SWPHP::_GetSubmodulos($modulo['permiso_id'], $modulo['clave'], basename($submodulo['clave']));
				foreach($subsubmodulos as $subsubmodulo)
				{
					$submodulo['hijos'][] = array(
						'textoenlace'=>$subsubmodulo['titulo'],
						'title'=>$subsubmodulo['descripcion'],
						'icono'=>$subsubmodulo['icono'],
						'estilo_id'=>'0',
						'estilo'=>'',
						'href'=>SWPHP_ADMIN_DIR.$subsubmodulo['clave'].'/',
						'abriraparte'=>'0',
						'on'=>false,
						'hijos'=>array(),
						);
				}

				$modulo['hijos'][] = array(
					'textoenlace'=>$submodulo['titulo'],
					'title'=>$submodulo['descripcion'],
					'icono'=>$submodulo['icono'],
					'estilo_id'=>'0',
					'estilo'=>'',
					'href'=>SWPHP_ADMIN_DIR.$submodulo['clave'].'/',
					'abriraparte'=>'0',
					'on'=>false,
					'hijos'=>array(),
					);
			}

			$menu[] = array(
				'textoenlace'=>$modulo['titulo'],
				'title'=>$modulo['descripcion'],
				'icono'=>$modulo['icono'],
				'estilo_id'=>'0',
				'estilo'=>'',
				'href'=>SWPHP_ADMIN_DIR.$modulo['clave'].'/',
				'abriraparte'=>'0',
				'on'=>false,
				'hijos'=>$modulo['hijos'],
				);
		}

		return $menu;
	}
	/**
	 * Obtiene los sub de un módulo, o los subsub de un submodulo
	 *
	 * @param string $mod_clave
	 * @param string $sub
	 * @return array
	 */
	static function _GetSubmodulos($permiso_id, $mod_clave, $sub='')
	{
		$submodulos = array();
		$base = SWPHP_MODULES_DIR.$mod_clave.'/admin/';
		if($sub)
		{
			$base .= "{$sub}/";
			$mod_clave .= "/{$sub}";
		}
		$dirs = getDirs($base);
		sort($dirs);
		foreach($dirs as $dir)
		{
			if(is_file($base.$dir.'/info.php'))
			{
				include($base.$dir.'/info.php'); //Aquí se define $submodulo

				if(!isset($submodulo['permiso_id']) || $permiso_id >= $submodulo['permiso_id'])
				{
					$submodulo['clave'] = $mod_clave.'/'.$dir;
					$submodulos[] = $submodulo;
				}
			}
		}

		return $submodulos;
	}
	/**
	 * Prepara los datos para insertar o actualizar datos genérico en idioma
	 *
	 * @param array $idiomas
	 * @param int $idioma_id
	 * @param array $campos
	 * @return array
	 */
	static function _PrepararDatosIdioma($idiomas, $idioma_id, $campos, $camposHtmlEntityDecode=array())
	{
		global $swphp_idiomas;

		$data = $idiomas[$idioma_id];

		foreach($data as $key=>$value)
		{
			if(in_array($key, $campos))
			{
				$data[$key] = trim(in_array($key, $camposHtmlEntityDecode) ? SWPHP::_HtmlEntityDecode($value) : $value);
				if($data[$key] == '' && $idioma_id != SWPHP_IDIOMA_POR_DEFECTO && !empty($idiomas[SWPHP_IDIOMA_POR_DEFECTO][$key]))
				{
					$data[$key] = SWPHP::_T($idiomas[SWPHP_IDIOMA_POR_DEFECTO][$key], SWPHP_IDIOMA_POR_DEFECTO_CODIGOISO, $swphp_idiomas[$idioma_id]['codigoiso']);
				}
			}
			else
			{
				//Cualquier otro dato lo descarto
				unset($data[$key]);
			}
		}

		return $data;
	}

	static function _Translate($str, $f=null, $t=null)
	{
		global $swphp_gtranslate;

		if(is_null($f))
		{
			$f = SWPHP_IDIOMA_POR_DEFECTO_CODIGOISO;
		}

		if(is_null($t))
		{
			$t = $_SESSION['SWPHP_IDIOMA_ACTUAL_CODIGOISO'];
		}

		if(empty($swphp_gtranslate))
		{
			global $swphp_config;

			if(!empty($swphp_config['idioma']['autotranslate']) && !empty($swphp_config['idioma']['googletranslateapikey']))
			{
				include_once(SWPHP_COMMON_DIR.'classes/MyCurl.class.php');
				include_once(SWPHP_COMMON_DIR.'classes/GoogleTranslate.class.php');

				$swphp_gtranslate = new GoogleTranslate($swphp_config['idioma']['googletranslateapikey']);
			}
			else
			{
				$swphp_gtranslate = -1;
			}
		}

		return is_object($swphp_gtranslate) ? utf8_decode_once($swphp_gtranslate->T(utf8_encode_once(trim($str)), $f, $t)) : $str;
	}

	static function _T($str, $f=null, $t=null)
	{
		return SWPHP::_Translate($str, $f, $t);
	}

	static function _Login($nick, $pass)
	{
		global $db;

		require_once SWPHP_COMMON_DIR.'classes/SWPHP_BanIP.class.php';

		$banip = new SWPHP_BanIP();
		if($banip->Check())
		{
			if($db->Exists(array($nick, $pass), array('nick', 'pass'), SWPHP_DBPREFIX."users"))
			{
				$user = $db->GetRow(sprintf("SELECT id, nick, email, perfil_id, activo, idioma_id FROM ".SWPHP_DBPREFIX."users WHERE nick = '%s'", addcslashes($nick,"'"), addcslashes($pass,"'")));

				if($user['perfil_id'] > 0 && $user['activo'])
				{
					$_SESSION['user'] = array(
						'id'=>$user['id'],
						'nick'=>$nick,
						'perfil_id'=>$user['perfil_id'],
						'email'=>$user['email'],
						'idioma_id'=>$user['idioma_id'],
						'autenticado'=>true,
						'recienautenticado'=>true,
						);

					SWPHP::_SetIdiomaActual($_SESSION['user']['idioma_id']);

					SWPHP::_Msg(sprintf("%s, <b>%s</b>", l('Bienvenido'), SWPHP::_HtmlEntities($nick)), "check");

					//Actualizar el contador de logins
					SWPHP::_ExecuteQuery(
						sprintf("UPDATE ".SWPHP_DBPREFIX."users SET contadorlogin = contadorlogin+1, ultimologin = '%s' WHERE id = '%s'", date("Y-m-d H:i:s"), $_SESSION['user']['id']),
						"",
						"",
						false,
						"error"
					);

					//Registrar el acceso
					SWPHP::_ExecuteQuery(
						sprintf("INSERT INTO ".SWPHP_DBPREFIX."login(id, user_id, fechayhora, ip) VALUES(NULL, '%u', '%s', '%s')", $_SESSION['user']['id'], date('Y-m-d H:i:s'), SWPHP_CLIENT_IP),
						"",
						"",
						false,
						"error"
						);

					global $swphp_administracion;

					//Indicar el módulo por defecto
					if($swphp_administracion && !isset($_GET['mod']))
					{
						$_GET['mod'] = SWPHP::_GetConfigValue('moduloadmin', 'configuracion_admin', false);
					}
				}
				else
				{
					SWPHP::_Msg(l('No autorizado'), 'prohibido');
					SWPHP::_Logout();
				}
			}
			else
			{
				$banip->Fallo('Login, datos de acceso incorrectos');
				SWPHP::_Msg(l('Datos de acceso incorrectos'), "error");
			}
		}
		else
		{
			SWPHP::_Msg(l('Acceso denegado por motivos de seguridad'), 'prohibido');
		}
	}

	static function _AdminModConfig($tabla='config', $conceptoPorDefecto=null)
	{
		global $swphp_mod_clave, $swphp_sub, $db, $smarty, $swphp_permiso_id, $swphp_module_templates_dir, $admin_template;

		if(is_null($conceptoPorDefecto))
		{
			$conceptoPorDefecto = $swphp_mod_clave;
		}

		$op = isset($_GET['op']) ? $_GET['op'] : '';
		$q = isset($_GET['q']) && !empty($_GET['q']) ? str_replace(array("\\/'\";"), "", $_GET['q']) : $conceptoPorDefecto;

		switch($op)
		{
			case 'cfg_apply':
				$conceptos = $db->GetValuesQuery("SELECT DISTINCT concepto FROM ".SWPHP_DBPREFIX."{$tabla} WHERE concepto LIKE '{$q}%'", false);
				foreach($conceptos as $concepto)
				{
					if(!isset($_POST[$concepto]))
					{
						$_POST[$concepto] = array(); //Tengo que hacer esto para los casos en que el concepto sólo tiene un parámetro y es un check
					}
				}

				foreach($_POST as $key=>$data)
				{
					if($db->Exists($key, "concepto", SWPHP_DBPREFIX.$tabla))
					{
						$checks = $db->GetValuesQuery("SELECT variable FROM ".SWPHP_DBPREFIX."config WHERE concepto = '{$key}' AND info LIKE '%type=checkbox%'");
						foreach($checks as $check)
						{
							$data[$check] = isset($data[$check]);
						}

						foreach($data as $i=>$v)
						{
							switch($i)
							{
								default:
									$data[$i] = trim($v);
									break;
							}
						}

						SWPHP::_SetConfigValues($data, $key, true, $tabla, $swphp_permiso_id < SWPHP_PERMISO_ACCESOTOTAL);
					}
				}
				SWPHP::_Msg("Se ha guardado la configuración", "check");
				//SWPHP::_Redirigir($swphp_config['url'].SWPHP_ADMIN_DIR);

				//sigue

			case 'cfg_show':
			default:
				include_once(SWPHP_COMMON_DIR."classes/FormBuilder.class.php");

				$smarty->assign("q", $q);
				$smarty->assign("ques", $swphp_mod_clave == 'configuracion' ? $db->GetRows("SELECT DISTINCT m.clave, m.titulo FROM ".SWPHP_DBPREFIX."modulos AS m JOIN ".SWPHP_DBPREFIX."{$tabla} AS c ON c.concepto = m.clave ORDER BY m.titulo") : false);

				$form = new FormBuilder();
				$form->SetAction(sprintf("%s%s%s/?%sop=cfg_apply", SWPHP_ADMIN_DIR, $swphp_mod_clave, empty($swphp_sub) ? '' : '/'.$swphp_sub, $q != $conceptoPorDefecto ? "q={$q}&" : ""));
				$form->SetMethod('post');
				$form->SetCancelUrl(sprintf("%s%s/", SWPHP_ADMIN_DIR, $swphp_mod_clave));
				$form->SetTextAccept("Guardar");
				$form->SetTextCancel("Cancelar");

				$smarty->assign("form_configuracion", $form->GetHTML(SWPHP::_GetConfigValuesToFormBuilder($q, $swphp_permiso_id, true, $tabla)));
				$admin_template = str_replace($swphp_mod_clave, 'configuracion', $swphp_module_templates_dir)."admin/configuracion.tpl";
				break;
		}
	}
	/**
	 * Aplica htmlentities con el quotestyle y charset establecidos
	 *
	 * @param string $str
	 * @return string
	 */
	static function _HtmlEntities($str, $quotestyle=SWPHP_QUOTESTYLE, $charset=SWPHP_CHARSET)
	{
		if(is_null($charset))
		{
			$charset = SWPHP_CHARSET;
		}

		return htmlentities($str, $quotestyle, $charset);
	}
	/**
	 * Aplica html_entity_decode con el quotestyle y charset establecidos
	 *
	 * @param string $str
	 * @return string
	 */
	static function _HtmlEntityDecode($str, $quotestyle=SWPHP_QUOTESTYLE, $charset=SWPHP_CHARSET)
	{
		if(is_null($charset))
		{
			$charset = SWPHP_CHARSET;
		}

		return html_entity_decode($str, $quotestyle, $charset);
	}

	static function _DescargarTablaEnCsv($tabla)
	{
		global $db;

		$filepath = SWPHP_TEMP_DIR.string2friendly($tabla).'_'.date('YmdHis').'.csv';
		$fp = fopen($filepath, 'w');

		$rs = DB_ResultSet::Factory($db->GetExtension());
		$rs->Set = $server->Execute("SELECT * FROM {$tabla}");
		$primero = true;
		while($reg = $rs->FetchAssoc())
		{
			if($primero)
			{
				fputcsv($fp, array_keys($reg), ';', '"');
				$primero = false;
			}

			fputcsv($fp, $reg, ';', '"');
		}

		fclose($fp);

		if($primero)
		{
			SWPHP::_Msg('No se encontraron registros en la tabla: '.$tabla, 'error');
		}
		else
		{
			SWPHP::_ForzarDescarga($filepath);
		}
	}

	static function _GetPoliticaPrivacidad()
	{
		global $db;

		$pagina_id = (int)SWPHP::_GetConfigValue('privacidad', 'contacto');

		return $pagina_id ? $db->GetValueQuery("SELECT contenido FROM ".SWPHP_DBPREFIX."paginas_idiomas WHERE pagina_id = '{$pagina_id}' AND idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}'", false) : false;
	}
	/**
	 * Crea un directorio si no existe, y opcionalmente mete dentro el index.php anti-listado en web de su contenido
	 *
	 * @param string $pathname
	 * @param bool $meterIndex
	 * @param int $mode
	 * @param bool $recursive
	 * @param resource $context
	 * @return boolean
	 */
	static function _MkDir($pathname, $meterIndex=true, $mode=0777, $recursive=false, $context=null)
	{
		if(is_dir($pathname))
		{
			if($meterIndex)
			{
				SWPHP::_MeterIndex($pathname);
			}
		}
		elseif($context ? mkdir($pathname, $mode, $recursive, $context) : mkdir($pathname, $mode, $recursive))
		{
			if($meterIndex)
			{
				SWPHP::_MeterIndex($pathname);
			}

			return true;
		}

		return false;
	}
	/**
	 * Mete en un directorio en index.php anti-listado en web de su contenido
	 *
	 * @param string $dir
	 */
	static function _MeterIndex($dir)
	{
		$aux = ponerBarra($dir).'index.php';
		if(!is_file($aux) || file_get_contents($aux) == '')
		{
			copy(SWPHP_COMMON_DIR.'index.php', $aux);
		}
	}
}
