<?php
/**
 * José Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2012-04-07
 *
 * SIMPLE WEB PHP EDICION ADMINISTRADA
 *
 * inc/modulos/kapitaltruth/admin/usuarios/functions.inc.php
 *
 */

/**
 * Valida los datos de un registro de un usuario
 *
 * @param array $data
 * @param int $id
 * @return bool
 */
function kapitaltruth_usuarios_validarDatos($data, $id=0)
{
	include_once(SWPHP_COMMON_DIR.'functions/validations.php');

	global $db;

	$ok = true;

	if($id && $db->GetValueQuery("SELECT u.perfil_id FROM ".SWPHP_DBPREFIX."users AS u JOIN ".SWPHP_DBPREFIX."kptl_usuarios AS ku ON ku.user_id = u.id WHERE ku.id = '{$id}'", false) >= $_SESSION['user']['perfil_id'])
	{
		$ok = false;
		SWPHP::_Msg(l("No tienes permiso para editar este usuario", 'kapitaltruth'), "prohibido");
	}
	else
	{
		$convertir = !$id && isset($data['convertir']) && $data['convertir'] ? intval($data['convertir']) : false;

		$requeridos = array(
			'nick'=>l('Alias', 'kapitaltruth'),
			'email'=>l('Email', 'kapitaltruth'),
			);

		if(!$id && !$convertir)
		{
			$requeridos['pass'] = l('Contraseña', 'kapitaltruth');
		}

		foreach($requeridos as $key=>$nombre)
		{
			if(!isset($data[$key]) || $data[$key] == '')
			{
				SWPHP::_Msg(l("Falta el dato:", 'kapitaltruth')." {$nombre}", "error");
				$ok = false;
			}
		}

		foreach($data as $key=>$value)
		{
			switch($key)
			{
				case 'nick':
					if(!$id && !$convertir && $db->Exists($value, 'nick', SWPHP_DBPREFIX.'users'))
					{
						$ok = false;
						SWPHP::_Msg(l('Ya existe un usuario con ese alias', 'kapitaltruth'), 'prohibido');
					}
					elseif(!validateString($value, SWPHP_NICK_MINLENGTH, SWPHP_NICK_MAXLENGTH, SWPHP_NICK_VALIDCHARS))
					{
						$ok = false;
						SWPHP::_Msg(l('Alias de usuario no válido', 'kapitaltruth'), 'error');
					}
					break;
				case 'convertir':
					if($convertir && !key_exists($value, kapitaltruth_usuarios_getConvertibles()))
					{
						$ok = false;
						SWPHP::_Msg(l('Usuario para convertir no válido', 'kapitaltruth'), 'error');
					}
					break;
				case 'pass':
					if($value != '')
					{
						if(!validateString($value, SWPHP_PASSWORD_MINLENGTH, SWPHP_PASSWORD_MAXLENGTH, SWPHP_PASSWORD_VALIDCHARS))
						{
							$ok = false;
							SWPHP::_Msg(l('Contraseña no válida', 'kapitaltruth'), 'error');
						}
						elseif($value != $data['cpass'])
						{
							$ok = false;
							SWPHP::_Msg(l('No has confirmado la contraseña correctamente', 'kapitaltruth'), 'error');
						}
					}
					break;
				case 'email':
					if(!validateEmail($value))
					{
						$ok = false;
						SWPHP::_Msg(l('Email no válido', 'kapitaltruth'), 'error');
					}
					elseif(!$id && !$convertir && $db->Exists($value, 'email', SWPHP_DBPREFIX.'users'))
					{
						$ok = false;
						SWPHP::_Msg(l('Ya existe un usuario con ese email', 'kapitaltruth'), 'prohibido');
					}
					break;
			}
		}
	}

	return $ok;
}
/**
 * Prepara los datos para insertar o actualizar un usuario en la tabla users
 *
 * @param array $data
 * @param int $id
 * @return array
 */
function kapitaltruth_usuarios_prepararDatosUser($data, $id=0)
{
	global $db;

	foreach($data as $key=>$value)
	{
		switch($key)
		{
			case "pass":
				$data[$key] = SWPHP::_Encrypt($value);
				break;
			case "nick":
			case "email":
				$data[$key] = trim($value);
				break;
			case "activo":
				$data[$key] = $value ? '1' : '0';
				break;
			default:
				//Cualquier otro dato lo descarto
				unset($data[$key]);
				break;
		}
	}

	$ahora = date('Y-m-d H:i:s');
	if($id == 0)
	{
		$data['perfil_id'] = 1; //Usuario registrado
		$data['fechayhoraregistro'] = $ahora;
		$data['user_idregistro'] = $_SESSION['user']['id'];
	}
	$data['fechayhoraultima'] = $ahora;
	$data['user_idultima'] = $_SESSION['user']['id'];

	return $data;
}
/**
 * Prepara los datos para insertar o actualizar un usuario en la tabla kptl_usuarios
 *
 * @param array $data
 * @param int $id
 * @return array
 */
function kapitaltruth_usuarios_prepararDatosUsuario($data, $id=0)
{
	include_once(SWPHP_COMMON_DIR.'functions/date.php');

    if(empty($data['grupos']))
    {
        $data['grupos'] = array();
    }

	foreach($data as $key=>$value)
	{
		switch($key)
		{
			case 'nombre':
				$data[$key] = trim($value);
				break;
			case 'fechanacimiento':
				$data[$key] = fechadmY2Ymd($value);
				break;
            case 'grupos':
                //Nada (pero no se descarta)
                break;
			default:
				//Cualquier otro dato lo descarto
				unset($data[$key]);
				break;
		}
	}

	if($id == 0)
	{
		$data["fechayhoraalta"] = date('Y-m-d H:i:s');
	}

	return $data;
}
/**
 * Elimina un usuario
 *
 * @param id $id
 */
function kapitaltruth_usuarios_eliminarUsuario($id)
{
	global $db;

	$user_id = $db->GetValue("user_id", SWPHP_DBPREFIX."kptl_usuarios", "WHERE id = '{$id}'");

	if($db->Exists($id, 'user_idactivo', SWPHP_DBPREFIX.'kptl_valoraciones')
		|| $db->Exists($id, 'user_idpasivo', SWPHP_DBPREFIX.'kptl_valoraciones')
		)
	{
		SWPHP::_Msg(l("No se puede eliminar por completo, tiene valoraciones realizadas y/o recibidas", 'kapitaltruth'), "prohibido");

		if($db->Execute("UPDATE ".SWPHP_DBPREFIX."users SET activo = 0 WHERE id = ".$user_id) && $db->AffectedRows() == 1)
		{
			SWPHP::_Msg(l("Se ha desactivado al usuario", 'kapitaltruth'));
            return -1;
		}
	}
	elseif($db->GetValue("perfil_id", SWPHP_DBPREFIX."users", "WHERE id = '{$user_id}'") > 1)
	{
		SWPHP::_Msg(l("No se puede eliminar, tiene perfil superior al de usuario registrado", 'kapitaltruth'), "prohibido");
	}
	elseif($db->Execute(buildQuery_Delete(SWPHP_DBPREFIX."kptl_usuarios", $id)))
	{
		if($db->AffectedRows() > 0)
		{
			$ok = true;

			$ok &= ($db->Execute("DELETE FROM ".SWPHP_DBPREFIX."users WHERE id = ".$user_id) !== false);
			kapitaltruth_limpiarTablas(); //$ok &= kapitaltruth_limpiarTablas();

			if($ok)
			{
				SWPHP::_Msg(l("Usuario eliminado correctamente", 'kapitaltruth'), "check");
			}
			else
			{
				SWPHP::_Msg(l("Usuario eliminado con errores", 'kapitaltruth'), "aviso");
			}

			return 1;
		}
	}
	else
	{
		SWPHP::_Msg(l("No se ha podido eliminar el usuario", 'kapitaltruth'), "error");
	}

	return 0;
}
/**
 * Obtiene un array asociativo (id=>nick) con los usuarios del sistema que el usuario actual puede convertir en usuarios de KPTL
 *
 * @return array
 */
function kapitaltruth_usuarios_getConvertibles()
{
	global $db;

	return $db->GetArrayForHTMLSelect('id', 'nick', SWPHP_DBPREFIX.'users', "WHERE perfil_id <= '{$_SESSION['user']['perfil_id']}' AND id NOT IN(SELECT user_id FROM ".SWPHP_DBPREFIX."kptl_usuarios)", array('',''));
}
