<?php
/*
 * 2013-04-02
 * José Carlos Cruz Parra
 * josecarlos@programadorphpfreelance.com
 * programadorphpfreelance.com
 *
 * ENVIADOR DE MENSAJES SISTEMA COMUNICACIÓN USUARIOS
 */

/**
 * Clase para enviar mensajes por distintos medios
 *
 * @author Jose Carlos Cruz Parra <josecarlos@programadorphpfreelance.com>
 *
 */
class SCU_Enviador
{
	private $_remitenteEmail = '';
	private $_remitenteEmailNombre = '';
	private $_remitenteSMS = ''; //Ej. 34689500011

	private $_movilespaises = array(
		'España'=>array(
			'prefijo'=>'34',
			'primerdigito'=>'6',
			),
		'Portugal'=>array(
			'prefijo'=>'351',
			'primerdigito'=>'9',
			),
		);

	/**
	 * Constructor
	 *
	 * @return SCU_Enviador
	 */
	function __construct()
	{
	}

	function SetRemitenteEmail($str)
	{
		$this->_remitenteEmail = $str;
	}

	function SetRemitenteEmailNombre($str)
	{
		$this->_remitenteEmailNombre = $str;
	}

	function SetRemitenteSMS($str)
	{
		$this->_remitenteSMS = $str;
	}

	/**
	 * Realiza un envío
	 *
	 * @param string $medio
	 * @param int $user_id
	 * @param string $asunto
	 * @param string $texto
	 * @param string $error
	 * @param string $destinatario
	 * @param int $mensaje_id
	 * @return bool
	 */
	public function EnviarMensaje($medio, $user_id, $asunto, $texto, $textocorto, &$error, &$destinatario, $mensaje_id)
	{
		global $db;

		switch($medio)
		{
			case 'EMAIL':
				$destinatario = $db->GetValue("email", SWPHP_DBPREFIX."users", "WHERE id = '{$user_id}'", true, false);
				if(strlen($destinatario) > 0)
				{
					if(validateEmail($destinatario))
					{
						$adjuntos = array();
						/*$adjuntos = $db->GetValuesQuery("SELECT DISTINCT filepath FROM scuadjuntos WHERE mensaje_id = '{$mensaje_id}'", false);
						if(!$adjuntos)
						{
							$adjuntos = array();
						}*/

						return $this->EnviarEmail($destinatario, $asunto, $texto, $error, $adjuntos);
					}
					$error = "Email no válido";
					return false;
				}
				$error = "Sin email";
				return false;
			case 'SMS':
				/*$destinatario = $db->GetValue("movil", "usuarios", "WHERE id_usuario = '{$user_id}'", true, false);
				$destinatario = str_replace(array('+34','+','-',' '), '', $destinatario);
				if(strlen($destinatario) > 0)
				{
					if(validatePhone($destinatario) && substr($destinatario,0,1) == '6')
					{
						return $this->EnviarSMS($destinatario, $textocorto, $error);
					}
					$error = "Móvil no válido";
					return false;
				}
				$error = "Sin móvil";*/
				$error = 'No está habilitado el envío por SMS';
				return false;
			case 'FAX':
				/*
				$destinatario = $db->GetValue("fax", "usuarios", "WHERE id_usuario = '{$user_id}'", true, false);
				$destinatario = str_replace(array('+34','+','-',' '), '', $destinatario);
				if(strlen($destinatario) > 0)
				{
					if(validatePhone($destinatario))
					{
						//TODO: convertir texto de html a texto plano
						return $this->EnviarFax($destinatario, $asunto.$texto, $error);
					}
					$error = "Fax no válido";
					return false;
				}
				$error = "Sin fax";
				*/
				$error = 'No está habilitado el envío por fax';
				return false;
			default:
				$error = "Medio no válido";
				return false;
		}
	}
	/**
	 * Realiza un envío por email
	 *
	 * @param string $destinatario
	 * @param string $asunto
	 * @param string $texto
	 * @param string $error
	 * @param array $adjuntos
	 * @return bool
	 */
	public function EnviarEmail($destinatario, $asunto, $texto, &$error, $adjuntos)
	{
		return SWPHP::_EnviarEmail($destinatario, $asunto, $texto, $adjuntos, $this->_remitenteEmail, $this->_remitenteEmailNombre, $error);
	}
	/**
	 * Realiza un envío por sms
	 *
	 * @param string $destinatario
	 * @param string $texto
	 * @param string $error
	 * @return bool
	 */
	public function EnviarSMS($destinatario, $texto, &$error)
	{
		$clickatell_config = SWPHP::_GetConfigValues('settings_clickatell', false);

		$api_id = $clickatell_config['api_id'];
		$user = $clickatell_config['user'];
		$password = $clickatell_config['password'];

		$movil = $this->_formatearMovil($destinatario);
		if($movil)
		{
			$query = sprintf('api_id=%s&user=%s&password=%s&to=%s&text=%s&from=%s&req_feat=48', $api_id, $user, $password, $movil, urlencode($this->_formatearTextoSMS($texto)), $this->_remitenteSMS);
			//echo $query.'<br />';
			$url = 'http://api.clickatell.com/http/sendmsg';

			$ch = curl_init($url.'?'.$query);
			//$ch = curl_init($url);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			//curl_setopt($ch, CURLOPT_POST, true);
			//curl_setopt($ch, CURLOPT_POSTFIELDS, urlencode($query));
			$html = curl_exec($ch);
			if(curl_errno($ch) > 0)
			{
				//Error de conexión
				$error = curl_errno($ch).": ".curl_error($ch);
				return false;
			}
			curl_close($ch);

			//Guardo la respuesta aunque no sea un error, para registrar el ID del envío
			$error = $html;

			//Procesar respuesta $html
			return ($html != '' && substr($html, 0, 3) != 'ERR');
		}
		else
		{
			//Número de móvil no válido
			$error = 'Móvil no válido '.$destinatario;
			return false;
		}

		//Por si acaso
		$error = 'Error desconocido';
		return false;
	}
	/**
	 * Realiza un envío por fax
	 *
	 * @param string $destinatario
	 * @param string $texto
	 * @param string $error
	 * @return bool
	 */
	public function EnviarFax($destinatario, $texto, &$error)
	{
		//TODO: enviar
		$error = "No está implementado el envío por fax";
		return false;
	}
	/**
	 * Formatea una cadena como número de móvil para enviarle un SMS. Devuelve false si no es un móvil correcto.
	 *
	 * @param string $str
	 * @return mixed
	 */
	private function _formatearMovil($str)
	{
		$movil = '';
		$strlen = strlen($str);
		for($c=0; $c<$strlen; $c++)
		{
			$char = substr($str, $c, 1);
			if(strpos('0123456789', $char) !== false)
			{
				$movil .= $char;
			}
		}
		$movillen = strlen($movil);

		foreach($this->_movilespaises as $key=>$movilpais)
		{
			$prefijolen = strlen($movilpais['prefijo']);
			$primerdigitolen = strlen($movilpais['primerdigito']);

			if(substr($movil, 0, $prefijolen) == $movilpais['prefijo'] && $movillen == $prefijolen + 9)
			{
				return $movil;
			}
			elseif(substr($movil, 0, $primerdigitolen) == $movilpais['primerdigito'] && $movillen == 9)
			{
				return $movilpais['prefijo'].$movil;
			}
		}

		return false;
	}
	/**
	 * Prepara una cadena para ser enviada como contenido de mensaje SMS
	 *
	 * @param string $str
	 * @return string
	 */
	private function _formatearTextoSMS($str)
	{
		$patrones = array(
			array(array('á','à','ä','â'), 'a'),
			array(array('é','è','ë','ê'), 'e'),
			array(array('í','ì','ï','î'), 'i'),
			array(array('ó','ò','ö','ô'), 'o'),
			array(array('ú','ù','ü','û'), 'u'),
			array(array('Á','À','Ä','Â'), 'A'),
			array(array('É','È','Ë','Ê'), 'E'),
			array(array('Í','Ì','Ï','Î'), 'I'),
			array(array('Ó','Ò','Ö','Ô'), 'O'),
			array(array('Ú','Ù','Ü','Û'), 'U'),
			array('ñ', 'n'),
			array('Ñ', 'N'),
			array('€', 'E'),
			array('$', 'S'),
			);

		foreach($patrones as $patron)
		{
			$str = str_replace($patron[0], $patron[1], $str);
		}

		return $str;
	}
}
?>