<?php
/**
 * Jose Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2011-03-18
 *
 * SIMPLE WEB PHP EDICION ADMINISTRADA
 *
 *  inc/plugins/multimedia/functions.inc.php
 */

/**
 * Muestra imágenes en miniatura ampliables, un reproductor de video flv, o un reproductor mp3,
 * según se indique por $ver, tomando los archivos correspondientes del directorio indicado por $dir
 *
 * @param string $ver
 * @param string $dir
 * @param int $itemsPerRow
 * @param int $thumbsWidth
 * @param int $thumbsHeight
 * @param bool $mostrarNombres
 * @param string $relkey
 * @param string $cropNombres
 * @param array $files
 * @return string
 */
function swphp_plugin_multimedia_multimedia($ver, $dir="multimedia/", $itemsPerRow=null, $thumbsWidth=null, $thumbsHeight=null, $mostrarNombres=null, $relkey=null, $cropNombres=null, $files=null)
{
	global $smarty;

	include_once(SWPHP_COMMON_DIR.'functions/images.php');
	include_once(SWPHP_MODULES_DIR.'multimedia/functions.inc.php');

	if(substr($dir, strlen($dir)-1) != '/')
	{
		$dir .= '/';
	}

	$thumbsdir = $dir.'thumbs/';
	SWPHP::_CheckDirWritable($thumbsdir);

	$cfg = SWPHP::_GetConfigValues("multimedia");

	if(is_null($itemsPerRow)) $itemsPerRow = $cfg["itemsPerRow"] ? $cfg["itemsPerRow"] : 4;
	if(is_null($thumbsWidth)) $thumbsWidth = $cfg["thumbsWidth"] ? $cfg["thumbsWidth"] : '200';
	if(is_null($thumbsHeight)) $thumbsHeight = $cfg["thumbsHeight"] ? $cfg["thumbsHeight"] : '150';

    if(is_null($mostrarNombres)) $mostrarNombres = isset($cfg["mostrarNombres"]) && $cfg["mostrarNombres"];
    if(is_null($cropNombres)) $cropNombres = isset($cfg["cropNombres"]) && $cfg["cropNombres"];

	$exts = array(
		'imagen'=>array('jpg','jpeg','gif','png','bmp'),
		'video'=>array('flv'),
		'audio'=>array('mp3'),
		);

	switch($ver)
	{
		case "imagen":
			if(is_null($files))
			{
				$files = getFilesExt($dir, $exts['imagen']);
				sort($files);
			}
			elseif(is_string($files))
			{
				$files = array($files);
			}

			$thumbs = array();
            $names = array();
			foreach($files as $file)
			{
				$thumbs[] = multimedia_getThumb($dir, $file, $exts, $thumbsWidth, $thumbsHeight);
                $name = getName($file);
                $names[] = trim($cropNombres && ($pos = strpos($name, $cropNombres)) !== false ? substr($name, $pos+1) : $name);
			}

			$smarty->assign("files", $files);
			$smarty->assign("thumbs", $thumbs);
            $smarty->assign("names", $names);

			$subtemplate = "multimedia_imagen.tpl";
			break;
		case "video":
			if(is_null($files))
			{
				$files = getFilesExt($dir, $exts['video']);
				sort($files);
			}
			elseif(is_string($files))
			{
				$files = array($files);
			}

			$filesexts = array();
            $urls = array();
			$thumbs = array();
			$names = array();
			foreach($files as $file)
			{
				$filesexts[] = getExtension($file, false);
                $urls[] = addcslashes(substr($file, 0, strrpos($file, ".")), "'");
				$thumbs[] = multimedia_getThumb($dir, $file, $exts, $thumbsWidth, $thumbsHeight);
                $name = getName($file);
                $names[] = trim($cropNombres && ($pos = strpos($name, $cropNombres)) !== false ? substr($name, $pos+1) : $name);
			}

			$smarty->assign("files", $files);
			$smarty->assign("filesexts", $filesexts);
            $smarty->assign("urls", $urls);
			$smarty->assign("thumbs", $thumbs);
            $smarty->assign("names", $names);

			$subtemplate = "multimedia_video.tpl";
			break;
		case "audio":
			if(is_null($files))
			{
				$files = getFilesExt($dir, $exts['audio']);
				sort($files);
			}
			elseif(is_string($files))
			{
				$files = array($files);
			}

			$strUrls = '';
			$strTitulos = '';
			foreach($files as $file)
			{
				if($strUrls != '')
				{
					$strUrls .= '|';
					$strTitulos .= '|';
				}
				$strUrls .= "{$dir}/{$file}";
				$strTitulos .= getName($file);
			}

			$smarty->assign("numFiles", sizeof($files));
		    $smarty->assign("strUrls", $strUrls);
		    $smarty->assign("strTitulos", $strTitulos);

		    $subtemplate = "multimedia_audio.tpl";
			break;
		default:
			return ""; //TODO: ¿mostrar un mensajito informativo?
	}

	if(is_numeric($thumbsWidth))
	{
		$thumbsWidth .= "px";
	}
	if(is_numeric($thumbsHeight))
	{
		$thumbsHeight .= "px";
	}

	$smarty->assign("dir", $dir);

	$smarty->assign('itemsPerRow', $itemsPerRow);
	$smarty->assign('thumbsWidth', $thumbsWidth);
	$smarty->assign('thumbsHeight', $thumbsHeight);

    $smarty->assign('mostrarNombres', $mostrarNombres);
    $smarty->assign('cropNombres', $cropNombres);

	$smarty->assign('multimedia_id', GenerateRandomKey(9));
    $smarty->assign("relkey", is_null($relkey) ? GenerateRandomKey(9) : $relkey);

	$smarty->assign('subtemplate', "../../../".SWPHP_MODULES_DIR."multimedia/tpl/{$subtemplate}");

	return $smarty->fetch("../../../".SWPHP_MODULES_DIR."multimedia/tpl/multimedia.tpl");
}
/**
 * Muestra las miniaturas ampliables de los archivos de imagen existentes en un directorio
 *
 * @param string $dir
 * @param int $itemsPerRow
 * @param int $thumbsWidth
 * @param int $thumbsHeight
 * @param bool $mostrarNombres
 * @param string $relkey
 * @param string $cropNombres
 * @param array $files
 * @return string
 */
function swphp_plugin_multimedia_imagen($dir="multimedia/", $itemsPerRow=null, $thumbsWidth=null, $thumbsHeight=null, $mostrarNombres=null, $relkey=null, $cropNombres=null, $files=null)
{
	return swphp_plugin_multimedia_multimedia("imagen", $dir, $itemsPerRow, $thumbsWidth, $thumbsHeight, $mostrarNombres, $relkey, $cropNombres, $files);
}
/**
 * Muestra un reproductor de video flv con miniaturas de los videos cargados de un directorio
 *
 * @param string $dir
 * @param int $itemsPerRow
 * @param int $thumbsWidth
 * @param int $thumbsHeight
 * @param bool $mostrarNombres
 * @param string $cropNombres
 * @param array $files
 * @return string
 */
function swphp_plugin_multimedia_video($dir="multimedia/", $itemsPerRow=null, $thumbsWidth=null, $thumbsHeight=null, $mostrarNombres=null, $cropNombres=null, $files=null)
{
	return swphp_plugin_multimedia_multimedia("video", $dir, $itemsPerRow, $thumbsWidth, $thumbsHeight, $mostrarNombres, null, $cropNombres, $files);
}
/**
 * Muestra un reproductor de audo mp3 con las canciones cargadas de un directorio
 * Los parámetros correspondientes a la presentación de las miniaturas no se usan (tal vez se usen en un futuro)
 *
 * @param string $dir
 * @param int $itemsPerRow
 * @param int $thumbsWidth
 * @param int $thumbsHeight
 * @return string
 */
function swphp_plugin_multimedia_audio($dir="multimedia/", $itemsPerRow=null, $thumbsWidth=null, $thumbsHeight=null)
{
	return swphp_plugin_multimedia_multimedia("audio", $dir, $itemsPerRow, $thumbsWidth, $thumbsHeight);
}
/**
 * Muestra la miniatura ampliable del archivo $path
 *
 * @param string $path
 * @param int $thumbsWidth
 * @param int $thumbsHeight
 * @param bool $mostrarNombres
 * @param string $relkey
 * @param string $cropNombres
 * @return string
 */
function swphp_plugin_multimedia_imagen_item($path, $thumbsWidth=null, $thumbsHeight=null, $mostrarNombres=null, $relkey=null, $cropNombres=null)
{
	return swphp_plugin_multimedia_imagen(ponerBarra(dirname($path)), 1, $thumbsWidth, $thumbsHeight, $mostrarNombres, $relkey, $cropNombres, $files=array(basename($path)));
}

function swphp_plugin_multimedia_flash_item($path, $width, $height)
{
	global $smarty;

	include_once(SWPHP_MODULES_DIR.'multimedia/functions.inc.php');

	$cfg = SWPHP::_GetConfigValues("multimedia");

	if(is_null($width)) $width = $cfg["thumbsWidth"] ? $cfg["thumbsWidth"] : '200';
	if(is_null($height)) $height = $cfg["thumbsHeight"] ? $cfg["thumbsHeight"] : '150';

    $smarty->assign('flashpath', $path);
	$smarty->assign('flashvars', '');
	$smarty->assign('width', $width);
	$smarty->assign('height', $height);

	return $smarty->fetch("../../../".SWPHP_MODULES_DIR."multimedia/tpl/multimedia_flash.tpl");
}
?>